!
! File is Kernel.f90
!
! This file controls the calls to the FORTRAN model
!
! OUTPUTS:
!
! created on 08.12.10
!
! david arthur sampson
! last write was: 03.06.12
! -----------------------------------------------
!
!
! =================================================================================================
Module lm_Initialize
 use gm_ModelControl
    !
    include 'C:\WaterSim\WaterSimTesting\WaterSimTesting\bin\Debug\App_Data\WaterSim_4_0\App_Data\Include\WaterSimDCDC.txt'
    !
 contains
    subroutine InitK()
     use lm_ParameterControl
     use lm_River_Aa
      use lm_Designations_A
      use lm_DesignationsDaily_A
     use lm_River_Ba
      use lm_Designations_B
     use lm_GPdemand
     use lm_Groundwater_A
     use lm_ProviderPopulation
     use lm_CitiWaterBudgets
     use lm_Meteorology
!     !
     call initParameters()
      call initMeteo()
       call initWSA()
        call initWSB()
         call initGP()
         call initGW()
         call initAdesignations()
        call initAdesignationsDaily()
       call initBdesignations()
      call initPop()
     call initCityWB()
     !
    end subroutine InitK        
    ! ------------------

    ! ---------------------
    subroutine InitStates()
    !
     use lm_River_Aa
     use lm_River_Ba
     use lm_GPdemand
     use lm_Groundwater_A
     use lm_ProviderPopulation
     use lm_Meteorology
     !
!     call initialState_a()
!     call initialState_b()
!     call initState_gp()
!     call initialState_g()
!     call initPop()
!     call initMeteo()
     !
    end subroutine InitStates
    ! -----------------------
!
End Module lm_Initialize
!
Module lm_Kernel
 use gm_GlobalData
  use gm_GlobalFiles
   use lm_BuildNotes
    !
    include 'C:\WaterSim\WaterSimTesting\WaterSimTesting\bin\Debug\App_Data\WaterSim_4_0\App_Data\Include\WaterSimDCDC.txt'
    !
 contains
    subroutine RunOneYearKernel()
     use gm_ModelControl
      use gm_TypeControl
        !
        integer(4) :: pop
        integer :: acft=0,year=2000
        integer(1) :: gvi=1
        logical :: gwo=.false.
        logical :: lvl_flag(gvi_maxProV)
        logical :: lvl_Interrupt=.false.
        logical :: lvl_OverRideInterrupt=.true.

        !--- TYPE constructs ---
        type(runTime)T
        type(RiverA)Aout
        type(RiverB)Bout
        type(Demand)D
        type(Provider)vPV
        type(Surfacewater)Sout
        type(Energy)EO
        ! ======================
        !
        if(gvi_kernel_case < 1)gvi_kernel_case=1
        call upStreamK(T,year)
         call globalDemand(T)                           ! calls calcDemand(T)
          call sCityParameterControl_a(T)
          !
          call sRiver_Aa(T,acft,Aout)                   ! Class A calculated here (WaterShed_SVT.f90)
          call sRiver_Ba(T,acft,Bout)            
          call sRivers(T,Aout,Bout,Sout)                ! Initial groundwater
          !
1         continue
          call sDemand(T,gvi)                          ! ProviderPopulationandDemand.f90
           call sVadose_s(T,gvi,lvl_flag)              ! Water_CityModel.f90
            call sDirectInject_s(T,gvi,lvl_flag)        ! " "
             call sReclaimed_k(T,gvi)                   ! " "
             call sROreclaimed_k(T,gvi,lvl_flag)        ! " "
             !
             if(.not. lvl_Interrupt)then
                call sGPdemand(T,D,pop,gwo,Sout)        ! WaterDemand_county.f90 line 630
                call sGroundwater(T,Sout,D,gwo)         ! Groundwater.f90 line 380
             endif
            !
          select case(gvi_kernel_case)
            case(1)
              call sSRP_classA(T,gvi,Aout)              ! SRPrelease.f90 line 334
              call sSRP_classBandCstorage(T,gvi,Aout,vPV)   ! Designations_SVT.f90 line 582
              call sCAP_water(T,gvi,Sout%sCAP_maf,vPV)  ! Designations_CO.f90 line 441
              call sSRP_NCS(T,gvi,Aout)                 ! SRPrelease.f90 line 390
              call sGWater(T,gvi)                       ! Groundwater.f90 line 413
              call sBanking_k(T,gvi,lvl_Interrupt)      ! Water_CityModel.f90 line 2731
              call sLakeHavasu(T,Bout,EO)
            case(2)
              call sSRP_classA(T,gvi,Aout)
              call sCAP_water(T,gvi,Sout%sCAP_maf,vPV)
              call sSRP_classBandCstorage(T,gvi,Aout,vPV)
              call sSRP_NCS(T,gvi,Aout)
              call sGWater(T,gvi)
              call sBanking_k(T,gvi,lvl_Interrupt)
              call sLakeHavasu(T,Bout,EO)
            case default
              call sSRP_classA(T,gvi,Aout)
              call sCAP_water(T,gvi,Sout%sCAP_maf,vPV)
              call sSRP_NCS(T,gvi,Aout)
              call sGWater(T,gvi)
              call sBanking_k(T,gvi,lvl_Interrupt)
          end select
         !
          if(lvl_OverRideInterrupt)then
            call sUnmetDemand(T,gvi,lvl_flag,lvl_Interrupt)
            ! Could use, instead, this exposed variable in the future: gvl_OverrideShortage
          else
            call sUnmetDemand(T,gvi,lvl_flag,lvl_Interrupt)
            if(lvl_Interrupt)then
              gvi=1
              goto 1
            endif
          endif
         !
         call sDesignate(T,Aout,Sout)
         call sWaterProviders(T,Sout,vPV)
         call sCitiModel(T,vPV,EO)
         call sSRPrelease(T,Aout,vPV)
         call sRiver_Ab(T,Aout)
        call downStreamK(T,year)
        gvi=1
        !
     return
    end subroutine RunOneYearKernel
    ! -----------------------------

    ! ---------------------------
     subroutine upStreamK(T,year)
      use gm_ModelControl
        !
        integer :: year
        integer :: run
        integer :: dt(8),lvi_buildNum, lvi_BuildVer
        integer(2) :: month,day,hour,minute
        character(10) :: c,d
        character(5) :: e,f,g,h
        type(runTime)T
      !
        gv_startyear=gpi_start    ! default
        if(gpl_validate)then
          gv_startyear=2000
        endif
        ! Override my default start year from the interface
       if(0 < gvi_startSimulationYear)then
        gv_startyear=gvi_startSimulationYear
       else
        gvi_startSimulationYear=gv_startyear
       endif
       !
        gv_endyear=gpi_end
       if(0 < gvi_endSimulationYear)then
        gv_endyear=gvi_endSimulationYear
       else
        gvi_endSimulationYear=gv_endyear
       endif
       gv_simduration=(gv_endyear-gv_startyear)
       !
       T%startyear=gv_startyear
       T%endyear= gv_endyear
       !
       T%simyear=gv_simyear
       if(gv_simduration == 0)gv_simduration=30
       if(year ==0 .or. year == gv_startyear .or. year > gv_startyear+gv_simduration-1)then
        if(gv_startyear ==0)then
          year=gpi_start
          gv_startyear=year
          T%simyear=0
        else
          year=gv_startyear
        endif
        if(year == gv_startyear)then
         T%startyear=gv_startyear
         T%year=T%startyear
         T%month=1
         T%simyear=0
         gv_simyear=0
         gvl_start=.true.
         !
         call ClearInclude(T)
         !
         call date_and_time(values=dt)
          lvi_buildNum=BuildNumK()
          lvi_BuildVer=BuildVerK()
          c = int2char(lvi_buildNum)
          d = int2char(lvi_BuildVer)
          ! Actual computer-based time
          month=dt(2)
          day=dt(3)
          hour=dt(5)
          minute=dt(6)
          !
          ! Run the validation sequence - 2000 to 2010
          ! Input from the interface 
          if(gvl_Validate)gpl_validate=.true.
          !
          if(.not. gpl_release)gvl_WriteToMyDisk=.true.
          ! Test version control date and time stamp
          e = int2char(MonthK())
          f = int2char(DayK())
          g = int2char(HourK())
          h = int2char(MinK())
         if(gvl_WriteToMyDisk)then
          write(7,*)"Today is:"," Month=",month,"Day=",day,"Hour=",hour,"Minute=",minute
          write(7,*)"Current Build # is: ",trim(c) // '.' // trim(d)
          write(7,*) " "
          write(7,*)"DLL Time Stamp: ",trim(e) //'.' // trim(f) // '.' // trim(g) // '.' // trim(h)
          write(7,*) "----------------------------------------"
          write(7,*)"Simulation start: upstream"
          write(7,*)" Once Through:"
         endif
        endif
      else
      endif  
      !
       run=gv_simduration
       T%simulations=run
      if(run == 0)T%simulations=40
      T%year=year
      call daysintheyearK(year,T)
      ! 
     return
    end subroutine upstreamK
   !------------------------

   ! ----------------------------
   subroutine downStreamK(T,year)
    use gm_ModelControl
    !
     integer :: year
     type(runTime)T
      !
      year=year+1
      T%year=year
      gv_simyear=gv_simyear+1
      T%simyear=gv_simyear
      gvl_start=.false.
      !
      if(year == gv_startyear+1)then 
        if(.not. gpl_release)gvl_WriteToMyDisk=.true.
        if(gvl_WriteToMyDisk)then
          write(7,*)"Simulation end: downstream"
          write(7,*)"----------------------------------------"
          write(7,*)"Run-time Errors (if present) as follows:"
          write(7,*) " "
        endif
      else if(year == T%endyear)then 
        year=gv_startyear 
        if(0 < gvi_startSimulationYear)year=gvi_startSimulationYear
      else
      endif
      !
   end subroutine downstreamK
   !-------------------------

    ! ----------------------------
    function int2char(i) result(c)
      integer, intent(in) :: i
      character(len=10) :: c
      write(c,'(i0)')i
    end function int2char
    ! -------------------

    ! -------------------------------
    subroutine daysintheyearK(year,T)
     use gm_ModelControl
 
        implicit none
       ! -----------------
        integer :: year
        integer LPyear

       !--- TYPE constructs ---
       type(runTime)T
       ! ======================
        !
        T%days=365
        LPyear=(int((year+1)/4)*4-1)+1
        if(year == LPyear)T%days=366
        !
       return
    end subroutine daysintheyearK
    ! ---------------------------

    ! -------------------
    subroutine startupK()

      character(len=200) :: lvc_outputPath=' '
        if(gpl_release)lvc_outputPath=trim(gvc_OPath)
        !
        if(.not. gpl_release)gvl_WriteToMyDisk=.true.
        !
        if(gvl_WriteToMyDisk)then
         gvl_WriteToDisk=.true.
        else
         gvl_WriteToDisk=.false.
        endif
        !
      if(gvl_WriteToMyDisk)then
        if(gpl_test)then
         open(2,FILE=trim(trim(lvc_outputPath)//'Two.txt'), status='replace')
         open(3,FILE=trim(trim(lvc_outputPath)//'Three.txt'), status='replace')
         open(100,FILE=trim(trim(lvc_outputPath)//'OneHundred.txt'), status='replace')
         open(101,FILE=trim(trim(lvc_outputPath)//'OneHundredOne.txt'), status='replace')

        endif
!        if(gpl_validate)then
         open(4,FILE=trim(trim(lvc_outputPath)//'Provider.txt'), status='replace')
         open(5,FILE=trim(trim(lvc_outputPath)//'NormalFlow.txt'), status='replace')
!        endif
         open(7,FILE=trim(trim(lvc_outputPath)//'Log.txt'), status='replace')
         open(8,FILE=trim(trim(lvc_outputPath)//'ProviderInputs.txt'), status='replace')
         open(9,FILE=trim(trim(lvc_outputPath)//'MaricopaInputs.txt'), status='replace')
      endif
      !
      if(gvl_WriteToMyDisk)then
        write(7,*)"Initialize (startupK - in Kernel.f90):"
        write(7,*)" "
        write(8,*)"WaterSIM dcdc: pls check the log file for the build version-", "Provider Inputs" 
        write(9,*)"WaterSIM dcdc: pls check the log file for the build version-", "Maricopa Inputs" 
      endif
      !
     return
    end subroutine startupK
    ! ---------------------

    ! --------------------
    subroutine teardownK()
     integer i
!      if(.not. gpl_release)gvl_WriteToMyDisk=.true.
      if(gvl_WriteToDisk)then
        if(gpl_test)then
          do i = 2,9,1
           close(i)
          end do
        else
         close(4)
         close(5)
         do i = 7,9,1
          close(i)
         end do
        endif
        close(100)
        close(101)
      endif
     return
    end subroutine teardownK
    ! ----------------------
!
End module lm_Kernel

Module lm_Start
 use lm_Kernel
 contains
  subroutine OpenFilesK()
   call startupK()
  end subroutine OpenFilesK
End Module lm_Start
!
Module lm_Terminate
 use lm_Kernel
 contains
  subroutine CloseFilesK()
   call teardownK()
  end subroutine CloseFilesK
End Module lm_Terminate
!
! =================================================================================================
! E.O.F

   subroutine test()
!     character(len=200) :: lvc_outputPath=' '
!      if(gpl_release)lvc_outputPath=gvc_outputPath
      !
!      if(.not. gv_start)open(102,FILE=trim(trim(lvc_outputPath)//'App_Data\Outputs\onehundredtwo.out'),status="replace")
!      gv_start=.true.
    end subroutine test

! ----------------------------------------------------------------------------------------
! E.O.F. really