!
! File is SRPrelease.f90
!
! This module estimates release from the SRP system of 6 reservoirs
!
! File created on: ?
!
!
! Last write was: 03.21.12
! =================================================================================================
Module lms_SRPrelease
 use gm_ModelControl
  use gm_GlobalData
   use gm_TypeControl
    use gm_Global

    implicit none
   ! -------------
    !
    include 'C:\WaterSim\WaterSimTesting\WaterSimTesting\bin\Debug\App_Data\WaterSim_4_0\App_Data\Include\WaterSimDCDC.txt'
    !
    type watershed_Ar
     real(8) :: lvd_ExpStorage_A_maf_yr(gpi_lBY:gpi_uBY)
     real(8) :: lvd_releaseExcess_A_maf_yr(gpi_lBY:gpi_uBY)
     real(8) :: gvd_allocateNCS_maf_yr(10)
     real(8) :: lvd_NCSmax_maf(10)
     real :: lvd_SRPrelease_maf_yr(gpi_lBY:gpi_uBY)
     real :: lvf_estSRPrelease_maf_yr(gpi_lBY:gpi_uBY)
    end type watershed_Ar
    !
   contains
   !
   !------------------------------------------------------
    ! 02.07.12
    !    Conservation space water and gatewater can be used off project. All other
    ! water including normal flow must be used on project.
 
   ! 02.02.12
    !   Water from the Reservoir and Pumped water  are treated the same, allocated based 
    ! on the 3 acre feet per acre.  SRP tries to deliver 3 acre feet per acre using a 
    ! combination of  water flow above normal flow rights, water from the reservoir, and
    ! pumped water.   From day to day how much each contributes to the 3 acre feet will 
    ! vary.  When there is not enough from these three to meet 3 acre feet, then SRP
    ! reduces the allocation it delivers to everyone.
 
    ! Ray Quay
    ! Research Professional
    ! Decision Center for a Desert City
    ! Global Institute of Sustainability
    ! Arizona State University
    ! Phone: 480.965.4394
    ! ==================================

         ! -------------------
         subroutine nRelease(T,Aout,BandC,wsa)
    
          integer(1) :: i
          real :: lvf_add_NCS,lvf_add_classA
          real :: lvf_checkState
          real :: lvf_state,lvf_flux,lvf_storage_flux,lvf_state,lvf_newState,lvf_diffState
          real(8) :: fEvaporation,vEvap_a,vFlow,lvd_NCSthresholdVolume
          real :: cvf_NCS(10),BandC

          real, parameter :: cvf_NCSVolume=0.272500
          data cvf_NCS(1),cvf_NCS(2),cvf_NCS(3),cvf_NCS(4),cvf_NCS(5),cvf_NCS(6) &
                ,cvf_NCS(7),cvf_NCS(8),cvf_NCS(9),cvf_NCS(10) /0,0.1,0,0.1,0.15,0,0.5,0.1,0.05,0/

          type(watershed_Ar):: wsa
          type(RiverA)Aout
          type(runTime)T
          ! ==============
              !
              lvd_NCSthresholdVolume=lv_storageCapacity-cvf_NCSVolume
              !
              vEvap_a=fEvaporation(lv_State_A_maf(T%year))
              !
              ! Scalers must use raw numbers
               vFlow=0.
              vFlow=lv_maf_yr(T%year)                         ! annual Flow (maf a-1)

              !
              ! BandC class A and estimated NCS from run-through
                !
                ! Estimated NCS- will be modified to create mass balance here
                !
                 lvf_add_NCS=0
                lvf_add_NCS=sum(Aout%gvf_SRP_NCS_AF_35)

                ! Normal Flow designations
                 lvf_add_classA=0
                lvf_add_classA=sum(Aout%gvf_SRP_classA_AF_35)

              !  Pass through reservoir
                vFlow=max(0,vFlow-(lvf_add_classA*gpd_acftTomaf))
                !
               lvf_storage_flux=0
              lvf_storage_flux=(BandC+lvf_add_NCS)*gpd_acftTomaf 
               lvf_flux=0
              lvf_flux=vFlow-lvf_storage_flux-vEvap_a
               lvf_newState =0
              lvf_newState=max(0,lv_State_A_maf(T%year) + lvf_flux)
               lvf_state=0
              lvf_state=lv_State_A_maf(T%year)
              !

              if(gpl_DemandDriven)then
                 wsa%lvd_releaseExcess_A_maf_yr(T%year)=0
                 wsa%lvd_SRPrelease_maf_yr(T%year)=0
                 !
                if( lvf_newState  <= li_deadpoolSVT)then
                  !
                  ! State at dead pool
                  ! ===================
                  if(0 < (vFlow-vEvap_a)) wsa%lvd_SRPrelease_maf_yr(T%year)=vFlow-vEvap_a
                   lv_State_A_maf(T%year)=li_deadpoolSVT
                  !
                else
                 lvf_diffState=0
                 lvf_checkState=0
                    !
                    if(lv_storageCapacity < lvf_newState)then
                       !
                       lv_State_A_maf(T%year)=lv_storageCapacity
                       !
                      if(lv_storageCapacity < lvf_flux + lvf_state)then
                        lvf_diffState=lvf_newState-lv_storageCapacity
                        lvf_checkState=(abs((lvf_state-lv_State_A_maf(T%year))/lv_State_A_maf(T%year)))*100
                        ! 
                        if(0.01 < lvf_checkState)then
                          wsa%lvd_releaseExcess_A_maf_yr(T%year)=max(0,lvf_flux-lvf_diffState)                   
                        else
                          ! At storage capacity
                          wsa%lvd_releaseExcess_A_maf_yr(T%year)=lvf_diffState
                        endif
                      else
                       wsa%lvd_releaseExcess_A_maf_yr(T%year)=0
                      endif
                      !
                       wsa%lvd_SRPrelease_maf_yr(T%year)= lvf_storage_flux + lvf_add_classA
                      !
                    else
                      !
                      lv_State_A_maf(T%year)=lvf_newState
                      wsa%lvd_SRPrelease_maf_yr(T%year)=lvf_storage_flux+lvf_add_classA
                      !                   
                    endif
                endif
              else
                  call basedOnsupply(lvd_NCSthresholdVolume,cvf_NCS,vFlow,vEvap_a,wsa)
              endif
              !
              go_StateSVT=NINT(lv_State_A_maf(T%year)*1/gpd_acftTomaf)
              go_spillageSVT=nint(wsa%lvd_releaseExcess_A_maf_yr(T%year)*1/gpd_acftTomaf)
              !
!write(3,30)T%year,lv_State_A_maf(T%year),vFlow,vFlow+lvf_add_classA*gpd_acftTomaf,wsa%lvd_releaseExcess_A_maf_yr(T%year),lvf_add_classA*gpd_acftTomaf,lvf_storage_flux-lvf_add_NCS*gpd_acftTomaf,lvf_add_NCS*gpd_acftTomaf

30      format(I4,1x,7(F10.7,1x))
              !
100          continue
            return
200         continue  
                if(gvl_WriteToMyDisk)write(7,*)"Error in SRPrelease.f90: flux < 0"
         end subroutine nRelease
         ! ---------------------

          ! -----------------------------------------------------------------------------------
          subroutine basedOnsupply(lvd_NCSthresholdVolume,cvf_NCS,lv_Modified_flow,vEvap_a,wsa)
            integer :: i
            real(8) :: vEvap_a
            real(8) :: lv_state,lv_deltaState,lv_Modified_flow
            real(8) :: lvd_NCSthresholdVolume
            real :: cvf_NCS(10)
            real, parameter :: cvf_NCSVolume=0.272500
            !
            type(watershed_Ar):: wsa
            type(runTime)T
            ! ===============
                !
                lv_deltaState=lv_Modified_flow-vEvap_a-wsa%lvd_SRPrelease_maf_yr(T%year) 
                lv_state=lv_state+lv_deltaState
                wsa%lvd_releaseExcess_A_maf_yr(T%year)=0
                !
                if(lv_state  >= li_deadpoolSVT)then
                  if(lv_state  >= lvd_NCSthresholdVolume)then
                     if(lv_state  >= lv_storageCapacity)then
                       do i = 1,10,1
                        wsa%gvd_allocateNCS_maf_yr(i)=cvf_NCSVolume*cvf_NCS(i)
                       end do    
                        if(lv_state-sum(wsa%gvd_allocateNCS_maf_yr)  >= lv_storageCapacity)then
                         lv_State_A_maf(T%year)=lv_storageCapacity
                        else
                         lv_State_A_maf(T%year)=lv_state-sum(wsa%gvd_allocateNCS_maf_yr)
                        endif
                     else
                       do i = 1,10,1
                          wsa%gvd_allocateNCS_maf_yr(i)=(lv_state-lvd_NCSthresholdVolume)*cvf_NCS(i)
                       end do     
                       lv_State_A_maf(T%year)= lvd_NCSthresholdVolume              
                     endif
                  else
                   lv_State_A_maf(T%year)=lv_state  
                  endif
                else
                  if(lv_State_A_maf(T%year)-vEvap_a-li_deadpoolSVT > 0)then
                   wsa%lvd_SRPrelease_maf_yr(T%year)=lv_State_A_maf(T%year)-vEvap_a-li_deadpoolSVT
                   lv_State_A_maf(T%year)=li_deadpoolSVT
                  else
                   wsa%lvd_SRPrelease_maf_yr(T%year)=0.
                   lv_State_A_maf(T%year)=li_deadpoolSVT
                  endif
                endif
            return
          end subroutine basedOnsupply
          ! --------------------------

          ! ---------------------------------
          subroutine classA(T,gvi_order,Aout,lvf_WaterDemand35)

            integer(1) :: gvi_order
            integer :: i,lvi_count
            real :: lvf_WaterDemand35(gvi_maxProV)
            !
            type(runTime)T
            type(RiverA)Aout
            ! ===============
              !
                 lvi_count=1
                  call sMaskSRP(gvl_maskSRP)
                if(gpl_DemandDriven)then
                  !
                  do i = 1,gvi_Providers,1
                    lvf_WaterDemand35(i)=0
                    gvf_UsedDemand_A_AF(i)=0
                   if(gvl_maskSRP(i))then      
                      ! changed on 05.18.11 according to RAY

!                     07.28.11 das 
                    !
                    ! 02.07.12 DAS on project demand (array block 3)
                    lvf_WaterDemand35(i)=gvf_WaterDemand_AF(i,gvi_order,3)*gpd_acftTomaf
                    !
                    if(lvf_WaterDemand35(i) <= Aout%gvd_ClassAarray_10_maf(lvi_count))then
                      gvf_UsedDemand_A_AF(i)=lvf_WaterDemand35(i)* (1./gpd_acftTomaf)
                    else
                      gvf_UsedDemand_A_AF(i)=Aout%gvd_ClassAarray_10_maf(lvi_count)* (1./gpd_acftTomaf)
                    endif

                    lvi_count=lvi_count+1
                   endif
                  end do  
                else
                  do i = 1,gvi_Providers,1
                   if(gvl_maskSRP(i))then                 
                      gvf_UsedDemand_A_AF(i)=Aout%gvd_ClassAarray_10_maf(lvi_count)* (1./gpd_acftTomaf)
                    lvi_count=lvi_count+1
                   endif
                  end do  
                endif
              !
          end subroutine classA
          ! --------------------

          ! ----------------------------------
          subroutine NCS(T,gvi_order,Aout,wsa)

            integer(1) :: gvi_order
            integer :: i,j,lvi_count
            real :: lvf_WaterDemand35(gvi_maxProV)
            real :: lvf_flow,vEvap_a

            real, parameter :: cvf_NCSVolume=0.272500
            real(8) :: lvd_NCSthresholdVolume
            real(8) :: fEvaporation
            real(8) :: lv_state,lv_deltaState,lvd_pureState

            real :: cvf_NCS(10), lvf_diffNCS(10)
            !
            ! ch=10%,gl=10%,me=15%,ph=50%,sc=10%,te=5%
            ! Maximum available=
            data cvf_NCS(1),cvf_NCS(2),cvf_NCS(3),cvf_NCS(4),cvf_NCS(5),cvf_NCS(6) &
                ,cvf_NCS(7),cvf_NCS(8),cvf_NCS(9),cvf_NCS(10) /0,0.1,0,0.1,0.15,0,0.5,0.1,0.05,0/

            type(watershed_Ar):: wsa
            type(RiverA)Aout
            type(runTime)T
            ! ===============
              !
                do i = 1,gvi_Providers,1
                  lvf_WaterDemand35(i)=0
                end do
                !
                 lvf_flow=0.
                lvf_flow=lv_maf_yr(T%year)
                !
                 vEvap_a=0
                 wsa%lvd_ExpStorage_A_maf_yr(T%year)=0
                vEvap_a=fEvaporation(lv_State_A_maf(T%year))
                wsa%lvd_ExpStorage_A_maf_yr(T%year)=lv_State_A_maf(T%year)+lvf_flow-vEvap_a
                !
                 lv_state=0
                lv_state=lv_State_A_maf(T%year)
                 lv_deltaState=0
                lv_deltaState=lvf_flow-(vEvap_a+sum(Aout%gvd_ClassAarray_10_maf))
                 lvd_pureState=0
                lvd_pureState=lv_state+lv_deltaState
                !
                lvd_NCSthresholdVolume=lv_storageCapacity-cvf_NCSVolume
                !
                do i = 1,10,1
                 wsa%gvd_allocateNCS_maf_yr(i)=0
                end do    
                !
                if(lvd_pureState  <= li_deadpoolSVT)then
                  !
                  ! State at dead pool
                  ! ===================
                  if(lv_State_A_maf(T%year)-vEvap_a-li_deadpoolSVT > 0)then
                   wsa%lvf_estSRPrelease_maf_yr(T%year)=lv_State_A_maf(T%year)-vEvap_a-li_deadpoolSVT
                   lv_State_A_maf(T%year)=li_deadpoolSVT
                  else
                   wsa%lvf_estSRPrelease_maf_yr(T%year)=0.
                   lv_State_A_maf(T%year)=li_deadpoolSVT
                  endif
                  !
                else
                  !
                  if(lvd_pureState  >= lvd_NCSthresholdVolume)then
                    if(lvd_pureState  >= lv_storageCapacity)then
                      do j = 1,10,1
                        wsa%gvd_allocateNCS_maf_yr(j)=0
                       wsa%gvd_allocateNCS_maf_yr(j)=cvf_NCSVolume*cvf_NCS(j)
                      end do
                    else
                      do i = 1,10,1
                       lvf_diffNCS(i)=0
                       lvf_diffNCS(i)=(lvd_pureState-lvd_NCSthresholdVolume)*cvf_NCS(i)
                       wsa%gvd_allocateNCS_maf_yr(i)=lvf_diffNCS(i)
                      end do

                    endif
                  else
                    do i = 1,10,1
                      wsa%gvd_allocateNCS_maf_yr(i)=0
                    end do
                  endif
                  !
                endif 
            !
                if(gpl_DemandDriven)then
                  !
                   call sMaskSRP(gvl_maskSRP)
                  lvi_count=1
                  do i = 1,gvi_Providers,1
                    gvf_Used_NCS_AF(i)=0
                    !
                    if(gvl_maskSRP(i))then     
                     ! 02.07.12 DAS
                     ! conservation space and gatewater can be used off-project
                     lvf_WaterDemand35(i)=gvf_WaterDemand_AF(i,gvi_order,1)*gpd_acftTomaf
                      ! Was this
!                     lvf_WaterDemand35(i)=gvf_WaterDemand_AF(i,gvi_order,3)*gpd_acftTomaf

                     if(lvf_WaterDemand35(i) < wsa%gvd_allocateNCS_maf_yr(lvi_count))then
                      gvf_Used_NCS_AF(i)=lvf_WaterDemand35(i)*1/gpd_acftTomaf
                     else
                      gvf_Used_NCS_AF(i)=wsa%gvd_allocateNCS_maf_yr(lvi_count)*1/gpd_acftTomaf
                     endif
                      wsa%gvd_allocateNCS_maf_yr(lvi_count)= gvf_Used_NCS_AF(i)*gpd_acftTomaf

                     lvi_count=lvi_count+1
                    endif
                  end do  
                else
                  do i = 1,gvi_Providers,1
                   if(gvl_maskSRP(i))then                 
                      gvf_Used_NCS_AF(i)=wsa%gvd_allocateNCS_maf_yr(lvi_count)*gpd_acftTomaf
                    lvi_count=lvi_count+1
                   endif
                  end do  
                endif
          end subroutine NCS
          ! -----------------
       
         ! -------------------------------
         subroutine aReleaseOut(T,wsa,Aout)
          use gm_TypeControl

            type(runTime)T
            type(watershed_Ar):: wsa
            type(RiverA)Aout
            ! ==============
              !
              Aout%gvd_SRP_release_maf_yr(T%year)=wsa%lvd_SRPrelease_maf_yr(T%year)
              Aout%gvd_releaseExcess_A_maf_yr(T%year)=wsa%lvd_releaseExcess_A_maf_yr(T%year)
              !
         end subroutine aReleaseOut

End Module lms_SRPrelease
    !
    subroutine sSRP_classA(T,gvi_order,Aout)
      use gm_Global
      use lms_SRPrelease
      !
      integer(1) :: gvi_order
      integer :: i,lvi_count
      !
      real :: lvf_WaterDemand35(gvi_maxProV)
      real :: lvf_on(gvi_maxProV),lvf_off(gvi_maxProV)
      real :: code=6

      type(RiverA)Aout
      type(runTime)T
      ! ===============
        !
        call classA(T,gvi_order,Aout,lvf_WaterDemand35)
        !
         call sMaskSRP(gvl_maskSRP)
         lvi_count=1
        do i = 1,gvi_Providers,1
          Aout%gvf_Unused_Class_A_AF(i)=0
          Aout%gvf_SRP_classA_AF_35(i)=0
          Aout%gvf_SRP_classA_AF_35(i)=gvf_UsedDemand_A_AF(i)
          ! 07.28.11
          if(gvl_maskSRP(i))then 
            Aout%gvf_Unused_Class_A_AF(i)=(max(0,(Aout%gvd_ClassAarray_10_maf(lvi_count)*1/gpd_acftTomaf) - gvf_UsedDemand_A_AF(i)))
            if(Aout%gvf_Unused_Class_A_AF(i) < 1)Aout%gvf_Unused_Class_A_AF(i)=0

           lvi_count= lvi_count+1
          endif     
  
        end do
          !
          ! ======================================================
          !
         ! New code NOW-07.28.11
          ! Proportionalize so that on-project gets a proportion of available

         ! New again-02.07.12, 02.09.12
         ! Only on-project for class A
         do i = 1,gvi_Providers,1
            !
            lvf_on(i)=gvf_UsedDemand_A_AF(i) 
            lvf_off(i)= 0. 
            !
         end do
          !
         call sCWaccounting(gvi_order,lvf_off,lvf_on,code)
         !
        gvi_order=gvi_order+1
        !
    end subroutine sSRP_classA
    ! ------------------------

    ! -----------------------------------
    subroutine sSRP_NCS(T,gvi_order,Aout)
      use gm_Global
      use lms_SRPrelease
      !
      integer(1) :: gvi_order
      integer :: i
      !
      real :: lvf_on(gvi_maxProV),lvf_off(gvi_maxProV)
      real :: lvf_proportionOn
      real :: code=9
      !
      type(RiverA)Aout
      type(runTime)T
      type(watershed_Ar):: ws
      ! =====================
        !
        call NCS(T,gvi_order,Aout,ws)
        !
        do i = 1,gvi_Providers,1
           Aout%gvf_SRP_NCS_AF_35(i)=0
          Aout%gvf_SRP_NCS_AF_35(i)=gvf_Used_NCS_AF(i)
          gvf_UsedDemand_A_AF(i)=gvf_Used_NCS_AF(i)
        end do
        !
        ! ==================================================
         !
         do i = 1,gvi_Providers,1
            lvf_proportionOn=0
          if(0 < gvf_WaterDemand_AF(i,gvi_order,1))then
           lvf_proportionOn= gvf_WaterDemand_AF(i,gvi_order,3)/gvf_WaterDemand_AF(i,gvi_order,1)
          endif
            !
            lvf_on(i)=gvf_UsedDemand_A_AF(i) *lvf_proportionOn
            lvf_off(i)= gvf_UsedDemand_A_AF(i)- lvf_on(i)
            !
         end do
         !
         call sCWaccounting(gvi_order,lvf_off,lvf_on,code)
         !
        gvi_order=gvi_order+1
        !
    end subroutine sSRP_NCS
    ! ---------------------

    ! --------------------------------
    subroutine sSRPrelease(T,Aout,vGP)
     use gm_Global
      use lms_SRPrelease
      !
      real :: BandC
      type(RiverA)Aout
      type(Provider)vGP

      type(runTime)T
      type(watershed_Ar):: ws
      ! ===============
        !

        ! 10 providers- BandC to Release is full provider list (maf)
         ! From Provider.f90- represents waterBanked
            !
            ! BCfromReservoir would also need to be reduced if we added a function
            ! to keep water in the river (for biological needs). 12.14.11 DAS
                ! Units are in AF for Aout%gvf_classBCfromReservoir
          BandC=0.
         BandC=sum(vGP%gvf_pAddToSRPrelease) +  sum(Aout%gvf_classBCfromReservoir)
        call nRelease(T,Aout,BandC,ws)
        call aReleaseOut(T,ws,Aout)
        !
    end subroutine sSRPrelease
! =================================================================================================
! E.O.F.
