!
! File is Water_CityModel.f90
!
! This file contains the code for modeling the water budgets of a city-provider based on the  
! work by  Chi Chi Choi and my implementation of that work.  Paul Westerhoff (Engineering)
!
! Global OUTPUTS:            
!
! Local OUTPUTS:
!   
! Local INPUTS:
!---------------------------------------------------
!
! created on 10.09.09
!
! david arthur sampson
! last write was: 03.05.12
! ------------------------
!
! =================================================================================================
Module lm_CitiWaterBudgets
!
 use gm_ModelControl
    use gm_GlobalData
   
    implicit none
    ! -------------
    !
    include 'C:\WaterSim\WaterSimTesting\WaterSimTesting\bin\Debug\App_Data\WaterSim_4_0\App_Data\Include\WaterSimDCDC.txt'
    !
! =================================================================================================
  contains
      subroutine initCityWB()
       call openFiles_wb()
       call readFiles_wb()
!       call initGW_wb()
      end subroutine initCityWB
      ! -----------------------

        ! -----------------------
        subroutine openFiles_wb()

          character(len=200) :: lvc_dataPath='App_Data\WaterSim_4_0\'
          if(gpl_release)lvc_dataPath=trim(gvc_DPath)
          !
           LU=65
          inquire(FILE=trim(trim(lvc_dataPath)//'App_Data\Data\agPumping_2085.txt'),exist=lexist)
          if(lexist)then
            open(65,FILE=trim(trim(lvc_dataPath)//'App_Data\Data\agPumping_2085.txt'),status='old')
          else
            if(gvl_WriteToDisk)then
             write(7,*)"Data in module lm_CitiWaterBudgets missing(or corrupted): agPumping_2085.txt", "Logical Unit = ", LU 
            endif
          endif
           LU=66
          inquire(FILE=trim(trim(lvc_dataPath)//'App_Data\Data\naturalRecharge_2085.txt'),exist=lexist)
          if(lexist)then
            open(66,FILE=trim(trim(lvc_dataPath)//'App_Data\Data\naturalRecharge_2085.txt'),status='old')
          else
            if(gvl_WriteToDisk)then
             write(7,*)"Data in module lm_CitiWaterBudgets missing(or corrupted): naturalRecharge_2085.txt", "Logical Unit = ", LU 
            endif
          endif
          !
         return
        end subroutine openFiles_wb
        ! ---------------------------

      ! -----------------------
      subroutine readFiles_wb()
        integer :: i,j,ios,LU
         real :: lvf_year,lvf_pumpingAg

          do i = gpi_minPVY,gpi_maxPVY,1
            read(65,*,err=10,iostat=ios)lvf_year,lvf_pumpingAg
            gv_AgWaterPumpingSRV(i,1)=lvf_pumpingAg
            lvf_year=lvf_year*1 ! Only used to surpress verbose warnings
          end do
10          continue
            close(65)
            if(0 < ios)then
             LU=65
             goto 1000
            endif
            !
            read(66,*,err=20,iostat=ios)((gvf_naturalRecharge(i,j),j=1,gvi_maxProV),i=gpi_lBY,gpi_uBY)
20          continue
            close(66)
            if(0 < ios)then
             LU=66
             goto 1000
            endif
         return
1000     continue
           if(gvl_WriteToDisk)&
            write(7,*)'Water_CityModel.f90 line 96: Error in file read, logical unit is', LU
      end subroutine readFiles_wb
      ! -------------------------
End Module lm_CitiWaterBudgets
!
Module lms_CitiWaterBudgets
 use gm_ModelControl
  use gm_GlobalData
   use gm_GlobalParms
    use gm_TypeControl

    implicit none
 
            real, private :: lv_gallonsPerFlush,lv_flushesPerDay,lvf_BWtypical
            real, private :: lvf_BWaterUsed,lvf_BWaterNotUsed,lvf_BWpct
            real, private :: gv_RateResLeak(gvi_maxProV)=0.0553  ! Mayer and DeOreo (1999), page 92-93 mean (median?)of total water use
            real, private :: gv_RateCioLeak(gvi_maxProV)=0.0553  ! Mayer and DeOreo (1999), page 92-93 = 9.5 gpcd/171.8 gpcd
            integer,private :: Use
            logical,public :: Citi_newYear
            logical, public :: lvl_firstLoop
            real,private :: gvf_EffluentPP,gvf_RegionalWWSurDischarge
            logical,public :: initParms

            ! Number of flux parameters for each state
            integer,private :: parms_Reclaimed=3,parms_Effluent=2
            integer,private :: parms_WWTP=2,parms_RWWTP=1
            integer,private :: parms_RO=1,parms_SW=2
            integer,private :: parms_WS=3
            !
            ! min and max fluxes

    ! -------------------------------------
    ! created locally
    ! ---------------
    ! State variables
    real, private :: vState_WB_CanalWater(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_SWTP(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_WaterSupply(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_WWTP(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

    real, private :: vState_WB_RWWTP(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_GWTP(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_ReclaimedWater(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_DirectInjection(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_Effluent(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    real, private :: vState_WB_GWB(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

    real, private :: vState_WB_GW(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    !
    real, private :: vState_WB_Residential(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
     real, private :: vState_WB_ResidentialIndoor(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
      real, private :: vState_WB_ResidentialOutdoor(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    !
    real, private :: vState_WB_CIOther(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
     real, private :: vState_WB_CIOtherIndoor(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
      real, private :: vState_WB_CIOtherOutdoor(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
    !
    real, private :: vState_WB_IReuse(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

    real, private :: vState_ROprocess(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

    ! ------------------------------------------------------------------------------------------
    !
    include 'C:\WaterSim\WaterSimTesting\WaterSimTesting\bin\Debug\App_Data\WaterSim_4_0\App_Data\Include\WaterSimDCDC.txt'
    ! ------------------------------------------------
    !
    type waterbalance
     real :: vGWtoAg(gvi_maxProV)=0.7

      real :: lvf_municipalPumping_AF_(gvi_maxProV)
       real :: lvf_municipalRecharge_AF_(gvi_maxProV)

      real :: lvf_demand_Res_outdoor_AF_(gvi_maxProV,gpi_timeStepWB)
       real :: lvf_demand_Res_indoor_AF_(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_demand_Cio_outdoor_AF_(gvi_maxProV,gpi_timeStepWB)
       real :: lvf_demand_Cio_indoor_AF_(gvi_maxProV,gpi_timeStepWB)
        real :: lvf_demand_AF(gvi_maxProV,gpi_timeStepWB)

      ! this will read Reclained_input 
      real :: lv_fluxFromWS_nonPotable(gvi_maxProV,gpi_timeStepWB)
      ! This is ROreclaimed water input
      real :: lv_fluxFromWS_Potable(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxFromWS_Groundwater(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxFromWS_SRPandCAP(gvi_maxProV,gpi_timeStepWB)

      real :: lv_fluxFromWS_Res_nonPotable(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxFromWS_Res_ROPotable(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxFromWS_Res_Groundwater(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxFromWS_Res_SRPandCAP(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxFromWS_CIO_nonPotable(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxFromWS_CIO_ROPotable(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxFromWS_CIO_Groundwater(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxFromWS_CIO_SRPandCAP(gvi_maxProV,gpi_timeStepWB)

         real :: lvf_fluxDemandDeficit(gvi_maxProV,gpi_timeStepWB)

      real :: lv_fluxFromWS_Res_Potable(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxFromWS_CIO_Potable(gvi_maxProV,gpi_timeStepWB)

     real :: lvf_fluxEffluentToAg(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_fluxEffluentToPowerPlant(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_fluxEffluentToRWSDischarge(gvi_maxProV,gpi_timeStepWB)

     real :: lv_ROflux_RO_Reclaimed(gvi_maxProV,gpi_timeStepWB)
      real :: lv_ROflux_DirectInject_Potable(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxFromSWTP(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_fluxFromSWTP_WS_acft(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxResidentialWW(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxResidential(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxResIndoor(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxResOutdoor(gvi_maxProV,gpi_timeStepWB)
     !
     real :: lv_fluxToRWWTP(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxRWWTPToReclaimed(gvi_maxProV,gpi_timeStepWB)

      real :: lvf_fluxReclaimedToVadose(gvi_maxProV,gpi_timeStepWB)
       real :: lvf_fluxReclaimedToDirectInject(gvi_maxProV,gpi_timeStepWB)
        real :: lvf_fluxReclaimedToOutput(gvi_maxProV,gpi_timeStepWB)
         real :: lvf_fluxReclaimedToRWWSDischarge(gvi_maxProV,gpi_timeStepWB)
          real :: lvf_fluxReclaimedDifference(gvi_maxProV,gpi_timeStepWB)
           real :: lvf_fluxReclaimedTotal(gvi_maxProV,gpi_timeStepWB)


      real :: lvf_fluxDirectInjectToGW(gvi_maxProV,gpi_timeStepWB)
       real :: lvf_fluxWSupplyToDirectInjectTotal(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxWWTPToEffluent(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxToRevOsmosis(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxWWToIRuse(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxToWWTPFromIRuse(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxWWToSurfaceDischarge(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxCioWW(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxCIO(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxCioIndoor(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxCioOutdoor(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxToVadoseFromEffluent(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxCioVadose(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxResVadose(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxVadoseToAquifer(gvi_maxProV,gpi_timeStepWB)
         real :: lv_fluxToVadoseFromAg(gvi_maxProV,gpi_timeStepWB)

     real :: lvf_fluxGWaterToGWaterTreatment(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_fluxGWaterTreatmentToWSupply(gvi_maxProV,gpi_timeStepWB)
       real :: lvf_fluxGWaterToWSupply(gvi_maxProV,gpi_timeStepWB)

     real :: lvf_fluxSWaterToGWaterBanking(gvi_maxProV,gpi_timeStepWB)
      real :: lvf_fluxGWaterBankingToSWater(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

     real :: lvf_fluxWStoDirectInjection(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxReclaimedGreywaterCIO(gvi_maxProV,gpi_timeStepWB)
      real :: lv_fluxReclaimedGreywaterRes(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxGWtoAgIrrigation(gvi_maxProV,gpi_timeStepWB)

      real :: lv_fluxSurfaceWaterToVadose(gvi_maxProV,gpi_timeStepWB)
       real :: lv_fluxSurfaceWaterToSWTP(gvi_maxProV,gpi_timeStepWB)
        real :: lv_fluxSurfaceWater_annual(gvi_maxProV)

      real :: lvf_fluxSWaterCAP(gvi_maxProV)
      real :: lvf_fluxSWaterSRP(gvi_maxProV)
        real ::  lvf_fluxSWaterSRP_NCS(gvi_maxProV),lvf_fluxSWaterSRP_A(gvi_maxProV)
      real :: lvf_fluxGWaterSRP(gvi_maxProV)
        real :: lvf_fluxBCstorageSRP(gvi_maxProV)


      real :: lv_fluxGWtoWTP(gvi_maxProV,gpi_timeStepWB)
       real :: lv_runoffPerc(gvi_maxProV,gpi_timeStepWB)
        real :: lvf_runoff(gvi_maxProV,gpi_timeStepWB)
         real :: lvf_rainfallEvaporation(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxAtmosTotal(gvi_maxProV,gpi_timeStepWB)

     real :: lv_fluxWWtoPaleoVerde(gvi_maxProV,gpi_timeStepWB)
     real :: lv_SRPtoEnergy(gpi_timeStepWB)

     real :: lv_providerPop(gvi_maxProV)

     real :: lvf_CAPuse_acft_month(gvi_maxProV,gpi_timeStepWB)

     real :: lvf_SRPclassBC_acft_month(gvi_maxProV,gpi_timeStepWB)
      !
     real :: lvf_TmaxResponse(gpi_timeStepWB)
    end type waterbalance
    !
    type parameters
     real :: In_1,In_2,In_3,In_4
     real :: Out_1,Out_2,Out_3,Out_4
    end type parameters
    !
  ! -------------------------------------------------------------------
! =================================================================================================
  contains
        subroutine sAquifer(T,vPV,lvWB,i,vTstep)

            integer :: i
            integer :: vTstep
            integer,parameter :: state=19

            real :: lv_AnnualWB(gvi_maxProV),lv_MonthlyWB(gvi_maxProV,gpi_timeStepWB)
            real :: lv_add(gvi_maxProV,gpi_timeStepWB),lv_sbt(gvi_maxProV,gpi_timeStepWB)
 
            type(Provider)vPV
            type (waterbalance)lvWB
            type(runTime)T
           ! ===============
             !
             call sMask(gvl_mask)
             !
             ! --------------
             call sRangeCheck(state,i,vTstep,gdf_parm_minGWtoGWT(i),gdf_parm_maxGWtoGWT(i),gvf_parm_GWtoGWTP(i),gvf_parm_GWtoGWTP(i))
             !
              ! Initial groundwater
              ! I initialize this in WaterShed.f90
             if(T%year == T%startyear)then
                vState_WB_GW(T%year,i,vTstep)=vState_WB_Aquifer(T%year,i)
             endif
             !
              ! variables sent in to this subroutine from Provider.f90
              ! -------------------------------------------------------
              !  monthly
              !  vPV%gvd_cGWrecharge_AF_monthly(i,vTstep)
              !  vPV%gvf_pMunicipalPumping_acft_monthly(i,vTstep)
              !  vPV%gvf_cSRPclassBC_AF_monthly(i,vTstep)=actual

              !  annual
              !  NO recharge
              !  lvWB%lvf_municipalPumping_AF_(i)
              !  lvWB%lvf_fluxGWaterSRP(i)

             ! ------------------------
             if(gpi_timeStepWB /=1)then
               
                ! From ProviderWater.f90 - recharge= designations > annual demand see lines 673-704
                lv_add(i,vTstep)=lvWB%lvf_municipalRecharge_AF_(i) &
                  + lvWB%lv_fluxVadoseToAquifer(i,vTstep) &
                  +  lvWB%lvf_fluxDirectInjectToGW(i,vTstep)
                !
                ! Under the demand formulation, municipal pmp
                lv_sbt(i,vTstep)=vPV%gvf_pMunicipalPumping_acft_monthly(i,vTstep) &
                  + vPV%gvf_cSRPclassBC_AF_monthly(i,vTstep)
                !
                lvWB%lvf_SRPclassBC_acft_month(i,vTstep)=vPV%gvf_cSRPclassBC_AF_monthly(i,vTstep)
                !
             else
                !

                lv_add(i,vTstep)= lvWB%lv_fluxVadoseToAquifer(i,vTstep) &
                  +  lvWB%lvf_fluxDirectInjectToGW(i,vTstep)
                !
                lv_sbt(i,vTstep)= lvWB%lvf_municipalPumping_AF_(i) &
                  +  lvWB%lvf_fluxGWaterSRP(i)
                !
                lvWB%lvf_SRPclassBC_acft_month(i,vTstep)= lvWB%lvf_fluxGWaterSRP(i)
                !
             endif
             !
              lvWB%lvf_fluxGWaterToGWaterTreatment(i,vTstep)=gvf_parm_GWtoGWTP(i)* lv_sbt(i,vTstep) 
             lvWB%lvf_fluxGWaterToWSupply(i,vTstep)= (1-gvf_parm_GWtoGWTP(i))* lv_sbt(i,vTstep) 
             !
              lv_MonthlyWB(i,vTstep)=0
              lv_AnnualWB(i)=0
             ! 
             if(gpi_timeStepWB /=1)then
              lv_MonthlyWB(i,vTstep)=lv_add(i,vTstep)-lv_sbt(i,vTstep)
              vState_WB_GW(T%year,i,vTstep)= vState_WB_GW(T%year,i,vTstep)+lv_MonthlyWB(i,vTstep)
             else
              lv_AnnualWB(i)=lv_add(i,vTstep)-lv_sbt(i,vTstep)
              vState_WB_Aquifer(T%year,i)=max(0,vState_WB_Aquifer(T%year,i)+lv_AnnualWB(i))
             endif
             !
             if(gpi_timeStepWB /=1)then
              ! NOT Implemented
              ! ================
              go_ProviderGroundWater_m(i,vTstep)=nint(lv_MonthlyWB(i,vTstep))
              !
             endif
             ! Ag groundwater pumped
!               lvf_Ag_In=0
!              lvf_Ag_In=gv_AgWaterPumpingSRV(T%year,1) 
              !
             call sUpdateState(T,i,vTstep,vState_WB_GW)
             call sUpdateState(T,i,vTstep,vState_WB_Aquifer)
             !
             !  Send an output to the interface- 08.20.10 das    
             if(vTstep == 12)then
               go_ProviderGroundWater_a(i)=nint(vState_WB_Aquifer(T%year,i))
              else
               go_ProviderGroundWater_a(i)=nint(vState_WB_Aquifer(T%year,i))
               goi_fluxVadoseToAquifer(i)=lvWB%lv_fluxVadoseToAquifer(i,vTstep)
              !
             endif
             !
             ! 
13        format(i4,1x,i2,1x,i2,1x,F5.3,1x,4(F8.1,1x),2x,F12.1)
          !
         return
        end subroutine sAquifer
        ! ---------------------

        ! ---------------------------------------
        subroutine sCommercialIO(T,lvWB,i,vTstep)

            integer :: i
            integer :: vTstep

            real :: lv_fluxWaterUseIn(gvi_maxProV,gpi_timeStepWB)
            real :: lv_providerGPCD(gvi_maxProV,2)
            real :: lv_fluxIndoorUse(gvi_maxProV,gpi_timeStepWB)
            real :: lv_fluxOutdoorUse(gvi_maxProV,gpi_timeStepWB)
            real :: lv_leakage(gvi_maxProV,gpi_timeStepWB)
            real :: lv_fluxToAtmos(gvi_maxProV,gpi_timeStepWB)

            real  :: vK1(gvi_maxProV)=0.207
            real  :: vK2(gvi_maxProV)=0.793    ! Evapotranspiration of outdoor water use

            real :: vGWtypical=0.535           ! 0.596  was 0.69 Avg prop of indoor water use contrib to grey water -Mayer and DeOreo (1999)
            !    
            real :: lvf_indoorDemand(gvi_maxProV,gpi_timeStepWB),lvf_outdoorDemand(gvi_maxProV,gpi_timeStepWB)
            real :: lvf_indoor_PotableMinusBlackWater

            real :: lvf_potable,lvf_nonpotable
            real :: lvf_indoor,lvf_indoor_deficit,lvf_indoorPotable
            real :: lvf_outdoor,lvf_outdoorNonpotable,lvf_outdoorPotable
            real :: lvf_nonPotable_useOutdoor,lvf_nonPotable_useIndoor
            !         
            type (waterbalance)lvWB
            type(runTime)T
            ! ===============
             !
              call sMask(gvl_mask)
              Use=1
             !
             ! NOW Five water sources: indoor,outdoor, ROreclaimed, and reclained, and groundwater
              ! -----------------------------------------------------------------------------------
              !
              ! -----------------------------------------
!             lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep)        - net water total
!             lvWB%lv_fluxFromWS_Cio_ROpotable(i,vTstep)    - ROreclaimed water
!             lvWB%lv_fluxFromWS_Cio_nonPotable(i,vTstep)   - reclaimed water
!             lvWB%lv_fluxFromWS_Cio_Groundwater(i,vTstep)  - Groundwater
!             lvWB%lv_fluxFromWS_Cio_Potable(i,vTstep)      - all potable

!             lvWB%lvf_demand_Cio_outdoor_AF_(i,vTstep)
!             lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep)
!             !
                lvf_indoorDemand(i,vTstep)=0
              lvf_indoorDemand(i,vTstep)=lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep)
              !
                lvf_outdoorDemand(i,vTstep)=0
              lvf_outdoorDemand(i,vTstep)=lvWB%lvf_demand_Cio_outdoor_AF_(i,vTstep)
              !
                lvf_nonpotable=0
              lvf_nonpotable=lvWB%lv_fluxFromWS_Cio_nonpotable(i,vTstep) 
              !
               lvf_potable=0
              lvf_potable=lvWB%lv_fluxFromWS_Cio_Potable(i,vTstep)                          
              ! ------------------------------------------------------------------------
              !
              ! =============================================================================================
               !
                lvf_nonPotable_useOutdoor=0
               lvf_nonPotable_useOutdoor=lvf_nonpotable*gvf_parm_ReclaimedOutdoor(i)

                lvf_nonPotable_useIndoor=0
               lvf_nonPotable_useIndoor=lvf_nonpotable-lvf_nonPotable_useOutdoor
               !
               call sBlackWater(lvWB,i,lvf_indoorDemand(i,vTstep),lvf_nonPotable_useIndoor, &
                  lvf_BWaterUsed,lvf_BWaterNotUsed,lvf_indoor_deficit)
               !
             ! ===============================================================================================
             ! time-step leakage
              lv_leakage(i,vTstep)=0
             lv_leakage(i,vTstep)= lvf_indoorDemand(i,vTstep)* gv_RateResLeak(i)
             if(.not. gpl_release) lv_leakage(i,vTstep)=0

             ! Use non-potable water for toilet first
               lvf_indoor_PotableMinusBlackWater=0
              lvf_indoor_PotableMinusBlackWater=anint((lvf_potable*(1-gvf_parm_OutDoorProp(i))) -lvf_BWaterUsed &
                - lv_leakage(i,vTstep))
               !
               lvf_indoor=0
              lvf_indoor=anint(lvf_indoor_PotableMinusBlackWater+lvf_BWaterUsed- lv_leakage(i,vTstep))
             !
               lvf_indoorPotable=0
              lvf_indoorPotable=lvf_potable*(1-gvf_parm_OutDoorProp(i))

              ! ===========================================
!              lvWB%lv_fluxSurfaceWaterToVadose(i,vTstep)   
!              lvWB%lvf_fluxSWaterToGWaterBanking(i,vTstep) 
!              lvWB%lvf_fluxWStoDirectInjection(i,vTstep) 
!
              if(gpi_timeStepWB /=1)then
              else           
                 lvf_outdoorNonpotable=0
                 lvf_outdoorPotable=0
                 lvf_outdoor=0
                !
                if(0 < lvf_nonpotable)then
                   !
                   lvf_outdoor=lvf_outdoorDemand(i,vTstep)
                   lvf_outdoorPotable=lvf_outdoorDemand(i,vTstep)-lvf_nonPotable_useOutdoor
                else
                  !
                  lvf_nonPotable_useOutdoor=0
                 lvf_outdoor=lvf_outdoorDemand(i,vTstep)
                 lvf_outdoorPotable=lvf_outdoor
                endif
                !
                lvf_outdoorNonpotable=lvf_nonPotable_useOutdoor
              endif
               !
if(gvl_mask(i))then
  if(gvl_WriteToDisk)then
    if(i == 33)then
    endif
  endif
endif
              ! 
              ! This should be zero if NO gray water is considered
              !~ 
              gvf_RateGreyWaterCIO(i)=0.
              !
              lv_fluxIndoorUse(i,vTstep)=lvf_indoor
              lv_fluxWaterUseIn(i,vTstep)=lvf_indoor+lvf_outdoor
              !
             if(T%year == T%startyear)then
               if(vTstep ==1)then
                lv_fluxReclaimedCioGreywater(T%year,i,vTstep)= &
                  lv_fluxIndoorUse(i,vTstep)*vGWtypical*gvf_RateGreyWaterCIO(i)
               endif
             endif
             !      
             lvWB%lv_fluxCIO(i,vTstep)= nint(lv_fluxWaterUseIn(i,vTstep))
             lvWB%lv_fluxCioIndoor(i,vTstep)= nint(lv_fluxIndoorUse(i,vTstep))
             !
             ! Outdoor use
             lv_fluxOutdoorUse(i,vTstep)=lvf_outdoor
             !
              ! ---------------------------------------------
              if(gvl_parm_grayWater)then
              call sGrayWater(T,lvWB,i,vTstep,lvf_indoor,lv_fluxWaterUseIn(i,vTstep), &
                lvf_BWaterUsed,lv_fluxReclaimedResGreywater,lv_providerGPCD)
              !
              lvWB%lv_fluxReclaimedGreywaterCio(i,vTstep)=(lv_fluxReclaimedCioGreywater(T%year,i,vTstep))
               !
              else
               lv_fluxReclaimedCioGreywater(T%year,i,vTstep)=0
              endif
              ! ---------------------------------------------
             !
             ! Outdoor water that moves into the vadose
             lvWB%lv_fluxCioVadose(i,vTstep)=nint(max(0.,(vK1(i)*lv_fluxOutdoorUse(i,vTstep))))
             lvWB%lv_fluxCioOutdoor(i,vTstep)=nint(lv_fluxOutdoorUse(i,vTstep)-lvWB%lv_fluxCioVadose(i,vTstep))

             ! Evaporative flux of outdoor water
              ! --------------------------------
              !call sRangeCheck(state,i,vTstep,min_vK1,max_vK1,vK1(i),vK1(i))
              !call sRangeCheck(state,i,vTstep,min_vK2,max_vK2,vK2(i),vK2(i))
               call sFluxToAtmos(lvWB,i,vTstep,vK1(i),vK2(i),lvWB%lv_fluxCioOutdoor(i,vTstep), lv_fluxToAtmos(i,vTstep))
              ! --------------------------------

             !
             ! -------- STATES ----------
             vState_WB_CIOther(T%year,i,vTstep)=vState_WB_CIOther(T%year,i,vTstep) &
              +(lv_fluxWaterUseIn(i,vTstep)+lv_fluxReclaimedCioGreywater(T%year,i,vTstep)) &
              -(lv_fluxIndoorUse(i,vTstep) + lv_fluxOutdoorUse(i,vTstep))
             !
             lvWB%lv_fluxCioWW(i,vTstep)=lv_fluxIndoorUse(i,vTstep) &
               -lv_fluxReclaimedCioGreywater(T%year,i,vTstep)
             !
            ! ====================== 
             vState_WB_CIOtherIndoor(T%year,i,vTstep)= vState_WB_CIOtherIndoor(T%year,i,vTstep) &
                +( lv_fluxIndoorUse(i,vTstep)+lv_fluxReclaimedCioGreywater(T%year,i,vTstep)) &
                - lvWB%lv_fluxCioWW(i,vTstep)
 
            vState_WB_CIOtherOutdoor(T%year,i,vTstep)=vState_WB_CIOtherOutdoor(T%year,i,vTstep) &
                + lv_fluxOutdoorUse(i,vTstep) &
                - (lvWB%lv_fluxCioVadose(i,vTstep) +  lv_fluxToAtmos(i,vTstep))
            !
            ! ---------------------
            ! Update states
            ! -----------------
             !
             call sUpdateState(T,i,vTstep, vState_WB_CIOtherIndoor)
             call sUpdateState(T,i,vTstep, vState_WB_CIOtherOutdoor)
             call sUpdateState(T,i,vTstep, vState_WB_CIOther)
            !
         return
        end subroutine sCommercialIO
        ! --------------------------

        ! ------------------------------------------
        subroutine sDirectInjection(T,lvWB,i,vTstep)
          integer :: i
           integer :: vTstep
           integer,parameter :: state=17

           type (waterbalance)lvWB
           type(runTime)T
           ! ===============
             !
            lvWB%lvf_fluxWSupplyToDirectInjectTotal(i,vTstep) = &
              lvWB%lv_ROflux_DirectInject_Potable(i,vTstep) &
              + lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep) &
              + lvWB%lvf_fluxWStoDirectInjection(i,vTstep)
            !
            lvWB%lvf_fluxDirectInjectToGW(i,vTstep)=  lvWB%lvf_fluxWSupplyToDirectInjectTotal(i,vTstep) 
            !
            vState_DirectInjection(T%year,i,vTstep)=vState_DirectInjection(T%year,i,vTstep) &
              +  lvWB%lv_ROflux_DirectInject_Potable(i,vTstep) &
              +  lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep) &
              +  lvWB%lvf_fluxWSupplyToDirectInjectTotal(i,vTstep) &
              -  lvWB%lvf_fluxDirectInjectToGW(i,vTstep)
            !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
            call sUpdateState(T,i,vTstep,vState_DirectInjection)
            !
         return
        end subroutine sDirectInjection
        ! -----------------------------
 
        ! -----------------------------------
        subroutine sEffluent(T,lvWB,i,vTstep)

           integer :: i
           integer :: vTstep
           integer,parameter :: state=16
           real  :: lv_fluxFromWWTP(gvi_maxProV,gpi_timeStepWB)
           !
           real  :: lv_fluxToPowerPlant(gvi_maxProV,gpi_timeStepWB)
           real  :: lv_fluxToVadose(gvi_maxProV,gpi_timeStepWB)
           real  :: lv_fluxToAgIrrigation(gvi_maxProV,gpi_timeStepWB)

!           ! For now, assume these fractions of waste water are sent to the 91st treatment plant
!            real  :: lv_WWfraction_21=0.89,lv_WWfraction_25=1.0,lv_WWfraction_30=1.0,lv_WWfraction_32=0.95

           type (waterbalance)lvWB
           type(parameters)P
           type(runTime)T
           ! ===============
             ! 
                 lv_fluxFromWWTP(i,vTstep)=0
                 lv_fluxToPowerPlant(i,vTstep)=0
                 lv_fluxToVadose(i,vTstep)=0
                 lv_fluxToAgIrrigation(i,vTstep)=0
              !
                P%In_1=gvf_parm_EffluentToVadose(i)
                P%In_2= gvf_parm_EffluentToPP(i)
                P%In_3=0
                P%In_4=0
               call sProportion(state,i,vTstep,parms_Effluent,initParms,P)
                gvf_parm_EffluentToVadose(i)=P%Out_1
                gvf_parm_EffluentToPP(i)=P%Out_2
              !

             ! Flux from the advanced waste water treatment plant
             lv_fluxFromWWTP(i,vTstep)= lvWB%lv_fluxWWTPToEffluent(i,vTstep)
              !
              ! These parameters will need checking in the interface
            ! to ensure that the sum <=1
               lv_fluxToPowerPlant(i,vTstep)=P%Out_2 * lv_fluxFromWWTP(i,vTstep)
               lv_fluxToVadose(i,vTstep)=P%Out_1 * lv_fluxFromWWTP(i,vTstep)
               !
               lv_fluxToAgIrrigation(i,vTstep)=lv_fluxFromWWTP(i,vTstep)&
                  -(lv_fluxToPowerPlant(i,vTstep)+lv_fluxToVadose(i,vTstep))
              !
              lvWB%lvf_fluxEffluentToPowerPlant(i,vTstep)= nint(lv_fluxToPowerPlant(i,vTstep))
              lvWB%lv_fluxToVadoseFromEffluent(i,vTstep)= nint(lv_fluxToVadose(i,vTstep))
              lvWB%lvf_fluxEffluentToAg(i,vTstep)= nint(lv_fluxToAgIrrigation(i,vTstep))
              !

              !
            ! ============== 
            vState_WB_Effluent(T%year,i,vTstep)=vState_WB_Effluent(T%year,i,vTstep) &
            + lv_fluxFromWWTP(i,vTstep) &
            - (lvWB%lv_fluxToVadoseFromEffluent(i,vTstep) &
            + lv_fluxToPowerPlant(i,vTstep) +  lv_fluxToAgIrrigation(i,vTstep))
            !
            lvWB%lv_fluxWWtoPaleoVerde(i,vTstep)=lv_fluxToPowerPlant(i,vTstep)
            !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
            call sUpdateState(T,i,vTstep,vState_WB_Effluent)
            !
         return
        end subroutine sEffluent
        ! ----------------------
 
        ! --------------------------------------
        subroutine sGWtreatment(T,lvWB,i,vTstep)
           integer :: i
           integer :: vTstep

           logical :: gvl_GWtreatmentPlant=.false.

           type (waterbalance)lvWB
           type(runTime)T
           ! ===============
             !
           if(gvl_GWtreatmentPlant)then
              ! Some loss or increase or effect takes place here
              lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep)=lvWB%lvf_fluxGWaterToGWaterTreatment(i,vTstep)
            else
              lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep)=0
            endif
             !
              vState_WB_GWTP(T%year,i,vTstep)= vState_WB_GWTP(T%year,i,vTstep) &
                +  lvWB%lvf_fluxGWaterToGWaterTreatment(i,vTstep) &
                - lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep)
            ! ---------------------
            ! Update states
            ! -----------------
            !
             call sUpdateState(T,i,vTstep,vState_WB_GWTP)
            !
           return
         end subroutine sGWtreatment
        ! --------------------------


        ! ------------------------------------------
        subroutine sIndustrialReuse(T,lvWB,i,vTstep)

           integer :: i,j,k
           integer :: vTstep
           real  :: lv_fluxToAtmos(gvi_maxProV,gpi_timeStepWB)

           real :: vK9(gvi_maxProV)=0.7014007467304

           type (waterbalance)lvWB
           type(runTime)T
           ! ===============
             !
              do j = 1,gvi_Providers,1
               do k = 1,gpi_timeStepWB,1
                 lv_fluxToAtmos(j,k)=0
               end do
              end do
             !
             ! Evaporative flux to the atmosphere
             lv_fluxToAtmos(i,vTstep)=(vK9(i)*lvWB%lv_fluxWWToIRuse(i,vTstep))
            if(0 < lv_fluxToAtmos(i,vTstep))then
             lvWB%lv_fluxAtmosTotal(i,vTstep)= lvWB%lv_fluxAtmosTotal(i,vTstep)+ lv_fluxToAtmos(i,vTstep)
            endif

             ! To waste water treatment plant
             lvWB%lv_fluxToWWTPFromIRuse(i,vTstep)=lvWB%lv_fluxWWToIRuse(i,vTstep)-lv_fluxToAtmos(i,vTstep)
             !
             ! ================
             vState_WB_IReuse(T%year,i,vTstep)=vState_WB_IReuse(T%year,i,vTstep) & 
               +lvWB%lv_fluxWWToIRuse(i,vTstep)-(lv_fluxToAtmos(i,vTstep) &
               +lvWB%lv_fluxToWWTPFromIRuse(i,vTstep))
             !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
             call sUpdateState(T,i,vTstep,vState_WB_IReuse)
            !
         return
        end subroutine sIndustrialReuse
        ! -----------------------------

        ! ------------------------------------
        subroutine sReclaimed(T,lvWB,i,vTstep)
            integer :: i
            integer :: vTstep    
            integer,parameter :: state=15
            real :: lvf_ReclaimedIn(gvi_maxProV,gpi_timeStepWB)
 
            type (waterbalance)lvWB    
            type(parameters)P
            type(runTime)T
            ! ===============
              !
              call sMask(gvl_mask)

              if(Citi_newYear)then
               gvf_RegionalWWSurDischarge=0
              endif
 
              lvf_ReclaimedIn(i,vTstep)=lvWB%lv_fluxRWWTPToReclaimed(i,vTstep)
              !
              ! -----------------------------
              !
                P%In_1=gvf_parm_RtoVadose(i)
                P%In_2=gvf_parm_RtoDInjection(i)
                P%In_3=gvf_parm_RtoOutput(i)
                P%In_4=0
              !
              if(gvl_parm_shortage(i))Then
                P%Out_1=gvf_parm_RtoVadose(i)
                P%Out_2=gvf_parm_RtoDInjection(i)
                P%Out_3=gvf_parm_RtoOutput(i)
              else
               call sProportion(state,i,vTstep,parms_Reclaimed,initParms,P)
                !
                gvf_parm_RtoVadose(i)=P%Out_1
                gvf_parm_RtoDInjection(i)=P%Out_2
                gvf_parm_RtoOutput(i)=P%Out_3
               !
              endif
                !
               lvWB%lvf_fluxReclaimedTotal(i,vTstep)        =lvf_ReclaimedIn(i,vTstep)
               lvWB%lvf_fluxReclaimedToVadose(i,vTstep)       =P%Out_1*lvf_ReclaimedIn(i,vTstep)
               lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep) =P%Out_2*lvf_ReclaimedIn(i,vTstep)
               lvWB%lvf_fluxReclaimedToOutput(i,vTstep)       =P%Out_3*lvf_ReclaimedIn(i,vTstep)
              !

              lvWB%lvf_fluxReclaimedToRWWSDischarge(i,vTstep)=lvf_ReclaimedIn(i,vTstep) &
                -  lvWB%lvf_fluxReclaimedToVadose(i,vTstep) &
                -  lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep) &
                -  lvWB%lvf_fluxReclaimedToOutput(i,vTstep) &
                +  lvWB%lvf_fluxReclaimedDifference(i,vTstep) 
              !
              ! lvf_fluxReclaimedDifference added on 09.02.11- was missing from balance: calculated in Water supply

              ! Sent to Water supply
              ! ----------------------
              vState_ReclaimedWater(T%year,i,vTstep)= vState_ReclaimedWater(T%year,i,vTstep) &
                +  lvf_ReclaimedIn(i,vTstep) - ( lvWB%lvf_fluxReclaimedToVadose(i,vTstep)&
                +  lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep)+ lvWB%lvf_fluxReclaimedToOutput(i,vTstep) &
                +  lvWB%lvf_fluxReclaimedToRWWSDischarge(i,vTstep))
              !
              gvf_RegionalWWSurDischarge=gvf_RegionalWWSurDischarge &
                + lvWB%lvf_fluxReclaimedToRWWSDischarge(i,vTstep) &
                + lvWB%lv_fluxWWToSurfaceDischarge(i,vTstep)              
              !
              gvf_reclaimedOutput(T%year,i,vTstep)= lvWB%lvf_fluxReclaimedToOutput(i,vTstep)
              !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
             call sUpdateState(T,i,vTstep,vState_ReclaimedWater)
             call sStepUpdate(T,i,vTstep,state)
            !
         return
        end subroutine sReclaimed
        ! -----------------------


        ! --------------------------------------
        subroutine sResidential(T,lvWB,i,vTstep)

            integer :: i
            integer :: vTstep

            real :: lv_fluxWaterUseIn(gvi_maxProV,gpi_timeStepWB)
            real :: lv_providerGPCD(gvi_maxProV,2)
            real :: lv_fluxIndoorUse(gvi_maxProV,gpi_timeStepWB)
            real :: lv_fluxOutdoorUse(gvi_maxProV,gpi_timeStepWB)
            real :: lv_leakage(gvi_maxProV,gpi_timeStepWB)
            real :: lv_fluxToAtmos(gvi_maxProV,gpi_timeStepWB)

            real :: vK1(gvi_maxProV)=0.207
            real :: vK2(gvi_maxProV)=0.793             ! Evapotranspiration of outdoor water use Chi Chi Cho/ Westerhoff
                                                     ! Reclaimed Greywater Res Rate =f(indoor avail-i.e. not toilet and leakage)
            real :: vGWtypical=0.535                 ! 0.596  was 0.69 Avg prop of indoor water use contrib to grey water -Mayer and DeOreo (1999)
            !
            real :: lvf_indoorDemand(gvi_maxProV,gpi_timeStepWB),lvf_outdoorDemand(gvi_maxProV,gpi_timeStepWB)
            real :: lvf_indoor_deficit 
            real :: lvf_indoor_PotableMinusBlackWater
            real :: lvf_outdoorNonpotable,lvf_outdoorPotable
            real :: lvf_potable,lvf_nonpotable,lvf_indoorPotable
            real :: lvf_nonPotable_useOutdoor,lvf_nonPotable_useIndoor

            real :: lvf_indoor
            real :: lvf_outdoor
            !         
            type (waterbalance)lvWB
            type(runTime)T
            ! ===============
              !
              call sMask(gvl_mask)

              ! NOW Five water sources: indoor,outdoor, ROreclaimed, and reclained, and groundwater
              ! -----------------------------------------------------------------------------------
              !
              Use=2
              ! 
              ! -----------------------------------------
!               lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep)        - net water total
!               lvWB%lv_fluxFromWS_Res_ROpotable(i,vTstep)    - ROreclaimed water
!               lvWB%lv_fluxFromWS_Res_nonPotable(i,vTstep)   - reclaimed water
!               lvWB%lv_fluxFromWS_Res_Groundwater(i,vTstep)  - Groundwater
!               lvWB%lv_fluxFromWS_Res_Potable(i,vTstep)      - all potable

!               lvWB%lvf_demand_Res_outdoor_AF_(i,vTstep)
!               lvWB%lvf_demand_Res_indoor_AF_(i,vTstep)
!               !
                lvf_indoorDemand(i,vTstep)=0
              lvf_indoorDemand(i,vTstep)=lvWB%lvf_demand_Res_indoor_AF_(i,vTstep)
              !
                lvf_outdoorDemand(i,vTstep)=0
              lvf_outdoorDemand(i,vTstep)=lvWB%lvf_demand_Res_outdoor_AF_(i,vTstep)
              !
                lvf_nonpotable=0
              lvf_nonpotable=lvWB%lv_fluxFromWS_Res_nonpotable(i,vTstep) 
              !
               lvf_potable=0
              lvf_potable=lvWB%lv_fluxFromWS_Res_Potable(i,vTstep)

               ! ------------------------------------------------------------------------
              !
if(gvl_WriteToMyDisk)then
if(i == 11)then
!write(5,*)T%year,lvf_potable,lvf_indoorDemand(i,vTstep)+lvf_outdoorDemand(i,vTstep)
endif
endif
              ! =============================================================================================
               !
                  lvf_nonPotable_useIndoor=0
                lvf_nonPotable_useIndoor= (1-gvf_parm_ReclaimedOutdoor(i))*lvf_nonpotable

                 lvf_nonPotable_useOutdoor=0
                lvf_nonPotable_useOutdoor=lvf_nonpotable-lvf_nonPotable_useIndoor
               !
               call sBlackWater(lvWB,i,lvf_indoorDemand(i,vTstep),lvf_nonPotable_useIndoor, &
                  lvf_BWaterUsed,lvf_BWaterNotUsed,lvf_indoor_deficit)
               !
             ! ===============================================================================================
             ! time-step leakage
              lv_leakage(i,vTstep)=0
             lv_leakage(i,vTstep)= lvf_indoorDemand(i,vTstep)* gv_RateResLeak(i)
             if(.not. gpl_release) lv_leakage(i,vTstep)=0
             ! Use non-potable water for toilet first
               lvf_indoor_PotableMinusBlackWater=0
              lvf_indoor_PotableMinusBlackWater= lvf_indoorDemand(i,vTstep)-lvf_BWaterUsed &
                - lv_leakage(i,vTstep)
               !
               lvf_indoor=0
              lvf_indoor=lvf_indoor_PotableMinusBlackWater+lvf_BWaterUsed-lv_leakage(i,vTstep)
             !
              lvf_indoorPotable=0
              lvf_indoorPotable=lvf_potable*(1-gvf_parm_OutDoorProp(i))

if(gvl_mask(i))then
  if(gvl_WriteToDisk)then
    if(i == 33)then
    endif
  endif
endif

              !
              ! ===========================================
!
!              feedback=lvf_SRPandCAPraw_Tplus1/ lvWB%lvf_cDemand_Res_total_AF_monthly(i,vTstep+1)
!              feedback=lvWB%lvf_cDemand_Res_total_AF_monthly(i,vTstep+1) &
!                  /lvWB%lvf_cDemand_Res_total_AF_monthly(i,vTstep) 
              !
              if(gpi_timeStepWB /=1)then
              else           
                 lvf_outdoorNonpotable=0
                 lvf_outdoorPotable=0
                 lvf_outdoor=0
                !
                if(0 < lvf_nonpotable)then
                   !
                   lvf_outdoor=lvf_outdoorDemand(i,vTstep)
                   lvf_outdoorPotable=lvf_outdoorDemand(i,vTstep)-lvf_nonPotable_useOutdoor
                else
                  !
                  lvf_nonPotable_useOutdoor=0
                 lvf_outdoor=lvf_outdoorDemand(i,vTstep)
                 lvf_outdoorPotable=lvf_outdoor
                endif
                !
                lvf_outdoorNonpotable=lvf_nonPotable_useOutdoor
              endif
              !
              lv_fluxIndoorUse(i,vTstep)=lvf_indoor
              lv_fluxWaterUseIn(i,vTstep)=lvf_indoor+lvf_outdoor
              !
              gvf_RateGreyWaterRES(i)=0.
             if(T%year == T%startyear)then
               if(vTstep ==1)then
                lv_fluxReclaimedResGreywater(T%year,i,vTstep)=lv_fluxIndoorUse(i,vTstep) &
                *vGWtypical*gvf_RateGreyWaterRES(i)
               endif
             endif
             !      
             lvWB%lv_fluxResidential(i,vTstep)=nint(lv_fluxWaterUseIn(i,vTstep))
             lvWB%lv_fluxResIndoor(i,vTstep)= nint(lv_fluxIndoorUse(i,vTstep))
             !
             ! Outdoor use
             lv_fluxOutdoorUse(i,vTstep)=lvf_outdoor
             !
              ! ---------------------------------------------
              if(gvl_parm_grayWater)then
              call sGrayWater(T,lvWB,i,vTstep,lvf_indoor,lv_fluxWaterUseIn(i,vTstep), &
                lvf_BWaterUsed,lv_fluxReclaimedResGreywater,lv_providerGPCD)
              !
              lvWB%lv_fluxReclaimedGreywaterRes(i,vTstep)=(lv_fluxReclaimedResGreywater(T%year,i,vTstep))
              !
               ! GPCD without graywater savings (i,1) and with graywater savings (i,2)
!               lvWB%lvf_GPCD_res(i,1)=lv_providerGPCD(i,1)
!               lvWB%lvf_GPCD_res(i,2)=lv_providerGPCD(i,2)
              else
                lv_fluxReclaimedResGreywater(T%year,i,vTstep)=0
              endif
              ! ---------------------------------------------
              !
             ! Outdoor water that moves into the vadose
             lvWB%lv_fluxResVadose(i,vTstep)=nint(max(0.,(vK1(i)*lv_fluxOutdoorUse(i,vTstep))))
             lvWB%lv_fluxResOutdoor(i,vTstep)= nint(lv_fluxOutdoorUse(i,vTstep)-lvWB%lv_fluxResVadose(i,vTstep))

             ! Evaporative flux of outdoor water
              ! --------------------------------
               call sFluxToAtmos(lvWB,i,vTstep,vK1(i),vK2(i),lvWB%lv_fluxResOutdoor(i,vTstep), lv_fluxToAtmos(i,vTstep))
              ! --------------------------------
            !
             ! States
             ! --------------------------------------
             vState_WB_Residential(T%year,i,vTstep)=vState_WB_Residential(T%year,i,vTstep) &
              +(lv_fluxWaterUseIn(i,vTstep)+lv_fluxReclaimedResGreywater(T%year,i,vTstep)) &
              -(lv_fluxIndoorUse(i,vTstep) + lv_fluxOutdoorUse(i,vTstep))
             !
             lvWB%lv_fluxResidentialWW(i,vTstep)=lv_fluxIndoorUse(i,vTstep) &
               -lv_fluxReclaimedResGreywater(T%year,i,vTstep)
             !
            ! ============== 
            vState_WB_ResidentialIndoor(T%year,i,vTstep)=vState_WB_ResidentialIndoor(T%year,i,vTstep) &
                +( lv_fluxIndoorUse(i,vTstep)+lv_fluxReclaimedResGreywater(T%year,i,vTstep)) &
                - lvWB%lv_fluxResidentialWW(i,vTstep)

             vState_WB_ResidentialOutdoor(T%year,i,vTstep)=vState_WB_ResidentialOutdoor(T%year,i,vTstep) &
                + lv_fluxOutdoorUse(i,vTstep) &
                - (lvWB%lv_fluxResVadose(i,vTstep) + lv_fluxToAtmos(i,vTstep))
            !
            ! ---------------------
            ! Update states
            ! -----------------
             !
             call sUpdateState(T,i,vTstep, vState_WB_ResidentialIndoor)
             call sUpdateState(T,i,vTstep, vState_WB_ResidentialOutdoor)
             call sUpdateState(T,i,vTstep, vState_WB_Residential)
            !
         return
        end subroutine sResidential
        ! -------------------------

        ! ------------------------------------
        subroutine sROprocess(T,lvWB,i,vTstep)
        
            integer :: i
            integer :: vTstep   
            integer,parameter :: state=14
!            integer :: gv_WWReuseControl=0 

            real :: lv_fluxFromRWWTP(gvi_maxProV,gpi_timeStepWB)
            real :: gv_ROremovalRate=0.95

!            real  :: vW3(gvi_maxProV)=0.20

            type (waterbalance)lvWB          
            type(parameters)P
            type(runTime)T
            ! ===============
              !
              call sMask(gvl_mask)

                P%In_1= gvf_parm_ROtoOutput(i)
                P%In_2=0
                P%In_3=0
                P%In_4=0
               !
              if(gvl_parm_shortage(i))Then
                P%Out_1=gvf_parm_ROtoOutput(i)
              else
               call sProportion(state,i,vTstep,parms_RO,initParms,P)
                gvf_parm_ROtoOutput(i)=P%Out_1
              end if
               !
               ! -------------------------------
                ! These need parameterization 05.02.11
                lvWB%lv_ROflux_RO_Reclaimed(i,vTstep)=0.
                lvWB%lv_ROflux_DirectInject_Potable(i,vTstep)=0.
                lv_fluxFromRWWTP(i,vTstep)=lvWB%lv_fluxToRevOsmosis(i,vTstep)
                ! 
                lvWB%lv_ROflux_RO_Reclaimed(i,vTstep)=gv_ROremovalRate* lv_fluxFromRWWTP(i,vTstep)
                !
                lvWB%lv_ROflux_DirectInject_Potable(i,vTstep)= &
                  (1.-gvf_parm_ROtoOutput(i))*lvWB%lv_ROflux_RO_Reclaimed(i,vTstep)
                !
                gvf_ROreclaimedOutput(T%year,i,vTstep)= gvf_parm_ROtoOutput(i)*lvWB%lv_ROflux_RO_Reclaimed(i,vTstep)
                !
                !
              vState_ROprocess(T%year,i,vTstep)=vState_ROprocess(T%year,i,vTstep) & 
                +  (gv_ROremovalRate* lv_fluxFromRWWTP(i,vTstep)) &
                -  lvWB%lv_ROflux_DirectInject_Potable(i,vTstep) &
                -  gvf_ROreclaimedOutput(T%year,i,vTstep)
            !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
            call sUpdateState(T,i,vTstep,vState_ROprocess)
            call sStepUpdate(T,i,vTstep,state)
            !
          return
        end subroutine sROprocess
        ! -------------------------

       ! ---------------------------------
        subroutine sRWWTP(T,lvWB,i,vTstep)
        
            integer :: i
            integer :: vTstep   
            integer,parameter :: state=13
            real :: lvf_checkWWtoRWWTP

            type (waterbalance)lvWB          
            type(parameters)P
            type(runTime)T
            ! ===============
              !
              if(lvl_firstLoop)call sMask(gvl_mask)
              !
                P%In_1= gvf_parm_RWWtoRO(i)
                P%In_2=0
                P%In_3=0
                P%In_4=0
               call sProportion(state,i,vTstep,parms_RWWTP,initParms,P)
                gvf_parm_RWWtoRO(i)=P%Out_1
               !
               ! -----------------------------------------------
                lvf_checkWWtoRWWTP=0
               lvf_checkWWtoRWWTP=gvf_parm_WWtoRWWTP(i)
               if(1 < lvf_checkWWtoRWWTP)gvf_parm_WWtoRWWTP(i)=1
               ! -----------------------------------------------
              !
              ! Reclaimed waste water treatmemt plant- 05.02.11 das
              lvWB%lv_fluxToRWWTP(i,vTstep)=gvf_parm_WWtoRWWTP(i)*(lvWB%lv_fluxResidentialWW(i,vTstep) &
                 + lvWB%lv_fluxCioWW(i,vTstep))

             ! Flux to RO
             lvWB%lv_fluxToRevOsmosis(i,vTstep)=P%Out_1 *  lvWB%lv_fluxToRWWTP(i,vTstep)

             !  Flux to 
             lvWB%lv_fluxRWWTPToReclaimed(i,vTstep)=lvWB%lv_fluxToRWWTP(i,vTstep) &
               - lvWB%lv_fluxToRevOsmosis(i,vTstep)               
             !
            ! ============== 
            vState_WB_RWWTP(T%year,i,vTstep)=vState_WB_RWWTP(T%year,i,vTstep) &
              + lvWB%lv_fluxToRWWTP(i,vTstep)-lvWB%lv_fluxToRevOsmosis(i,vTstep)
             !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
            call sUpdateState(T,i,vTstep,vState_WB_RWWTP)
            !
         return
        end subroutine sRWWTP
        ! -------------------

        ! -------------------------------------------
        subroutine sSurfaceWater(T,vPV,lvWB,i,vTstep)

           integer :: i
           integer :: vTstep
           integer,parameter :: state=1

           real :: gvf_fluxFromGWBanking(gvi_maxProV,gpi_timeStepWB)
           real :: lvf_SWaterToVadose(gvi_maxProV),lvf_SWaterToBanking(gvi_maxProV)

           type (waterbalance)lvWB
           type(Provider)vPV
           type(runTime)T
           ! ===============
            !
            if(lvl_firstLoop)then
              call sMask(gvl_mask)
              call sMaskCAP(gvl_maskCAP)
            endif
            ! ------------------------
            ! --------------------------------
             !
             ! CAP and SRP 
              lvWB%lv_fluxSurfaceWater_annual(i)= &
                lvWB%lvf_fluxSWaterCAP(i)+ lvWB%lvf_fluxSWaterSRP(i)+ lvWB%lvf_fluxGWaterSRP(i)
              !
              if(gpi_timeStepWB /=1)then
                if(vTstep == 1)then
                else
                endif
              else
                  gvf_fluxFromGWBanking(i,vTstep)= gvf_GW_Banking(T%year,i,1)
              endif
              !
              lvWB%lvf_fluxGWaterBankingToSWater(T%year,i,vTstep)=gvf_fluxFromGWBanking(i,vTstep)
              !
              if(gvl_maskCAP(i))then
                 lvf_SWaterToVadose(i) =0
                lvf_SWaterToVadose(i) =  (gvf_parm_SWtoVadoseAmt(i)) /gpi_timeStepWB

                ! 
                 lvf_SWaterToBanking(i)=0               
                lvf_SWaterToBanking(i)=vPV%gvf_cSetWBanking_a(i)
                !
                 lvWB%lv_fluxSurfaceWaterToVadose(i,vTstep)   =  lvf_SWaterToVadose(i)
                lvWB%lvf_fluxSWaterToGWaterBanking(i,vTstep) =  lvf_SWaterToBanking(i)
                !
                ! To the surface water treatment plant
                lvWB%lv_fluxSurfaceWaterToSWTP(i,vTstep)= lvWB%lv_fluxSurfaceWater_annual(i) &
                  - (lvWB%lv_fluxSurfaceWaterToVadose(i,vTstep) &
                  + lvWB%lvf_fluxSWaterToGWaterBanking(i,vTstep))
              ! 
             endif
            ! ---- STATE -----
            !
            vState_WB_CanalWater(T%year,i,vTstep)=nint(vState_WB_CanalWater(T%year,i,vTstep) &
              +  lvWB%lv_fluxSurfaceWater_annual(i) +  gvf_fluxFromGWBanking(i,vTstep) &
              -  lvWB%lv_fluxSurfaceWaterToVadose(i,vTstep) &
              -  lvWB%lvf_fluxSWaterToGWaterBanking(i,vTstep) &
              -  lvWB%lv_fluxSurfaceWaterToSWTP(i,vTstep))
            !
            ! ---------------------
            ! Update states
            ! -----------------
            !
            call sUpdateState(T,i,vTstep,vState_WB_CanalWater)
            !       
             return
100       continue
          stop
        end subroutine sSurfaceWater
        ! --------------------------

  
        ! -------------------------------
        subroutine sSWTP(T,lvWB,i,vTstep)

           integer :: i
           integer :: vTstep
           real :: lv_fluxToSWTP(gvi_maxProV,gpi_timeStepWB)
           integer,parameter :: state=3

!           real :: lv_evaporativeLoss=0.04  ! Assume 4% loss at the SWTP

           type (waterbalance)lvWB
           type(runTime)T
           ! ===============
             !
             ! Has GW banking and SW to Vadose removed- has indoor and outdoor fluxes
              lv_fluxToSWTP(i,vTstep)=0
             lv_fluxToSWTP(i,vTstep)= lvWB%lv_fluxSurfaceWaterToSWTP(i,vTstep)
             !
             lvWB%lvf_fluxFromSWTP_WS_acft(i,vTstep)= lv_fluxToSWTP(i,vTstep)
            !
            ! ============== 
            vState_WB_SWTP(T%year,i,vTstep)=vState_WB_SWTP(T%year,i,vTstep)+ &
             lv_fluxToSWTP(i,vTstep)-lvWB%lvf_fluxFromSWTP_WS_acft(i,vTstep)
            !
            ! ---------------------
            ! Update states
            ! -----------------
            !
             call sUpdateState(T,i,vTstep,vState_WB_SWTP)
            !
           return
          end subroutine sSWTP
        ! --------------------

        ! -------------------------------------
        subroutine sVadose(T,lvWB,i,vTstep)

           integer :: i
           integer :: vTstep
           integer :: lvi_TimeLag
           !    
           real  :: lv_fluxToVadose(gvi_maxProV,gpi_timeStepWB)  
           real :: lvf_lagFluxToVadose(gpi_lBY:gpi_uBY,gvi_maxProV)
           integer,parameter :: state=18
 
           type (waterbalance)lvWB
           type(runTime)T
           type localVadose
            real :: SW,Effluent,RunoffPerc
            real :: Ag,OutdoorCIOandRes
            real :: Recharge
            real :: NaturalRecharge
            !
            real :: LagFlux
           end type localVadose
           type(localVadose) V
           ! ===============
            !
            lvi_TimeLag=gpi_Tlag
            if(0 < gvi_timeLagVadose(i))lvi_TimeLag=gvi_timeLagVadose(i)
             !~ ------------------------------------------------------------------
            !  This is from SRP and CAP surface water
             V%SW=0
            V%SW        =lvWB%lv_fluxSurfaceWaterToVadose(i,vTstep)
             V%Effluent=0
            V%Effluent  =lvWB%lv_fluxToVadoseFromEffluent(i,vTstep) &
                + lvWB%lvf_fluxReclaimedToVadose(i,vTstep)
             V%RunoffPerc=0
            V%RunoffPerc=lvWB%lv_runoffPerc(i,vTstep)

              ! Ag is left out of individual municipal water providers
             V%Ag=0
            V%Ag        =lvWB%lv_fluxToVadoseFromAg(i,vTstep)
             V%OutdoorCIOandRes=0
            V%OutdoorCIOandRes=  lvWB%lv_fluxCioVadose(i,vTstep)+lvWB%lv_fluxResVadose(i,vTstep)
             V%Recharge=0
            V%Recharge  =lvWB%lvf_municipalRecharge_AF_(i)

             V%NaturalRecharge=0
            V%NaturalRecharge=gvf_naturalRecharge(T%year,i)
            ! --------------------------------------------------
            !
            call sHistoricalVadose(T,lvi_TimeLag,lvWB,i,vTstep,V)
            !
            if(initParms)V%LagFlux=0
            !
            lv_fluxToVadose(i,vTstep)=  V%SW + V%Effluent +  V%RunoffPerc &
              + V%OutdoorCIOandRes + V%Recharge + V%NaturalRecharge
            !
            ! ----------------------------
            !
            if(T%year < T%startyear+lvi_TimeLag)then
             lvf_lagFluxToVadose(T%year,i)= gvf_lagFluxVadoseToAquifer(T%year,i,vTstep)
            else
             lvf_lagFluxToVadose(T%year,i)= max(0,nint(lv_fluxToVadose(i,vTstep)))
            endif
            !
            lvWB%lv_fluxVadoseToAquifer(i,vTstep)= lvf_lagFluxToVadose(T%year,i)
             !
!             ! -- STATE ----
!            ! ============== 
            vState_WB_Vadose(T%year,i,vTstep)= vState_WB_Vadose(T%year,i,vTstep) &
             +  lv_fluxToVadose(i,vTstep) - lvWB%lv_fluxVadoseToAquifer(i,vTstep)
            !
3 format(I4,1x,I2,1x,6(F12.3,2x))
            ! ---------------------
            ! Update states
            ! -----------------
            !
             call sUpdateState(T,i,vTstep,vState_WB_Vadose)
            !
          return
        end subroutine sVadose
        ! --------------------
    
          ! ---------------------------------------------------------
          subroutine sHistoricalVadose(T,lvi_TimeLag,lvWB,i,vTstep,V)
            integer :: i,j,k,l
            integer :: vTstep
            integer :: lvi_TimeLag

            real :: lv_popMinus10(gvi_maxProV),lv_popDiff10(gvi_maxProV)
            real  :: lv_addOther(gvi_maxProV,gpi_timeStepWB),lv_rateOther(gvi_maxProV,gpi_timeStepWB)
            real  :: lv_rateR(gvi_maxProV,gpi_timeStepWB),lv_Rminus10(gvi_maxProV,gpi_timeStepWB) &
                    ,lv_slopeR(gvi_maxProV,gpi_timeStepWB)
            real  :: lv_Rminus10other(gvi_maxProV,gpi_timeStepWB),lv_slopeOther(gvi_maxProV,gpi_timeStepWB)
            real  :: lv_approximateOther(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
            real  :: lv_approximateRecharge(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
            real :: lvf_GrowthRatePop(gpi_lBY:gpi_uBY,gvi_maxProV)
            !
            type localVadose
             real :: SW,Effluent,RunoffPerc
             real :: Ag,OutdoorCIOandRes
             real :: Recharge
             real :: NaturalRecharge
             !
             real :: LagFlux
            end type localVadose

            type(localVadose)V
            type (waterbalance)lvWB
            type(runTime)T
            ! ===============
            !
            if(T%year == T%startyear)then
              lv_addOther(i,vTstep)=0
              lv_rateOther(i,vTstep)=0
              lv_slopeOther(i,vTstep)=0
              lv_rateR(i,vTstep)=0
              lv_Rminus10(i,vTstep)=0
              lv_Rminus10other(i,vTstep)=0
              lv_slopeR(i,vTstep)=0
              !
                 lvf_GrowthRatePop=li_providerpop
              !
            endif
!            !
            if(T%year == T%startyear)then
              if(gpi_timeStepWB /= 1)then
                if(i == 1 .and. vTstep == 1)then
                  do j = 1,gvi_Providers,1
                    lv_popMinus10(j)=0
                    lv_popDiff10(j)=0
                  end do
                  do j = gpi_lBY,gpi_uBY,1
                    do k = 1,gvi_Providers,1
                      do l = 1,12,1
                        gvf_lagFluxVadoseToAquifer(j,k,l)=0
                      end do
                    end do
                  end do
                endif
              else
                if(i == 1)then
                  do j = 1,gvi_Providers,1
                    lv_popMinus10(j)=0
                    lv_popDiff10(j)=0
                  end do
                 do j = gpi_lBY,gpi_uBY,1
                    do k = 1,gvi_Providers,1
                        gvf_lagFluxVadoseToAquifer(j,k,1)=0
                    end do
                  end do
                endif
    
              endif
              if(vTstep == 1)then
                 !
                lv_popDiff10(i)=lvf_GrowthRatePop(T%year+lvi_TimeLag,i)-lvf_GrowthRatePop(T%year,i)
                if(lvf_GrowthRatePop(T%year,i)- lv_popDiff10(i) > 0)then
                  lv_popMinus10(i)=lvf_GrowthRatePop(T%year,i)- lv_popDiff10(i)
                else
                  lv_popMinus10(i)=0
                endif
                !
              endif
              if(V%Recharge > 0)then
                if(0 < lvf_GrowthRatePop(T%year,i) )then
                  lv_rateR(i,vTstep)= V%Recharge/lvf_GrowthRatePop(T%year,i)
                endif
                  lv_Rminus10(i,vTstep)=lv_rateR(i,vTstep)*lv_popMinus10(i)
                  if(0 < lvi_TimeLag)lv_slopeR(i,vTstep)=(V%Recharge-lv_Rminus10(i,vTstep) )/lvi_TimeLag
              endif
                !
                lv_addOther(i,vTstep)=nint(max(0.,V%Effluent + V%OutdoorCIOandRes &
                   + V%SW + V%RunoffPerc + V%NaturalRecharge + V%Ag))
                !
              if(0 < lvf_GrowthRatePop(T%year,i))then
                lv_rateOther(i,vTstep)=(lv_addOther(i,vTstep)/lvf_GrowthRatePop(T%year,i))
                lv_Rminus10other(i,vTstep)=(lv_rateOther(i,vTstep)*lv_popMinus10(i))
                if(0 < lvi_TimeLag)lv_slopeOther(i,vTstep)=nint((lv_addOther(i,vTstep)-lv_Rminus10other(i,vTstep))/lvi_TimeLag)
              else
               if(gvl_WriteToDisk)&
                write(7,*)T%year,"Population is at zero in City Water Budgets, line 1438, for provider: ",i
              endif
              !
                if(T%year+lvi_TimeLag < 2050)then
                  do j = 1,lvi_TimeLag+1,1
                    lv_approximateRecharge(T%year+j-1,i,vTstep)=max(0.,nint( lv_Rminus10(i,vTstep)+(lv_slopeR(i,vTstep)*(j-1))))
                    lv_approximateOther(T%year+j-1,i,vTstep)=max(0., nint(lv_Rminus10other(i,vTstep)+ (lv_slopeOther(i,vTstep)*(j-1))))
                    !
                    gvf_lagFluxVadoseToAquifer(T%year+j-1,i,vTstep)=max(0.,(lv_approximateOther(T%year+j-1,i,vTstep) &
                      + lv_approximateRecharge(T%year+j-1,i,vTstep)))
                    !
                  end do
                endif
                !
                vState_WB_Vadose(T%year,i,vTstep)=max(0,gvf_lagFluxVadoseToAquifer(T%startyear,i,vTstep)*lvi_TimeLag)
              !
            endif
           return
          end subroutine sHistoricalVadose
          ! ------------------------------

       ! -----------------------------------
        subroutine sWSupply(T,lvWB,i,vTstep)
        
           integer :: i
           integer :: vTstep
           integer,parameter :: state=4
           real :: lv_fluxToWS(gvi_maxProV,gpi_timeStepWB)
           !
           type typeWS
            real :: lvf_Reclaimed,lvf_ROreclaimed
           end type typeWS
           type(typeWS) lvWS
           type (waterbalance)lvWB
           type(parameters)P
           type(runTime)T
           ! ============
            !
            if(lvl_firstLoop)then
              call sMask(gvl_mask)
              call sMaskCAP(gvl_maskCAP)
            endif
            !
             P%In_1= gvf_parm_WStoRes_Prop(i)
             P%In_2= gvf_parm_WStoCio_Prop(i)
             P%In_3=0
             P%In_4=0
             call sProportion(state,i,vTstep,parms_WS,initParms,P)
             gvf_parm_WStoRes_Prop(i)=P%Out_1
             gvf_parm_WStoCio_Prop(i)=P%Out_2
             !
             ! FromSWTP has been corrected for WBanking and Vadose storage
             !
             ! If a shortage occurs, defined in subroutine unmetdemand, then no water sent to DI
!             lvWB%lvf_fluxWStoDirectInjection(i,vTstep) =  (gvf_parm_WStoDI(i)*gvf_parm_WStoDIamount(i)) /gpi_timeStepWB
             lvWB%lvf_fluxWStoDirectInjection(i,vTstep) =  (gvf_parm_WStoDIamount(i)) /gpi_timeStepWB

             ! Must look at demand and supply here to balance water
            ! -----------------------------------------------------
            if(gpi_timeStepWB /= 1)then
              call sBalanceWS(T,lvWB,i,vTstep,lvWS)
            else
              call sBalanceWS(T,lvWB,i,vTstep,lvWS)
            endif
            ! -----------------------------------------------------
             !
              lv_fluxToWS(i,vTstep)=0
             lv_fluxToWS(i,vTstep)= anint(lvWB%lvf_fluxFromSWTP_WS_acft(i,vTstep) & 
              + lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep) &
              + lvWB%lvf_fluxGWaterToWSupply(i,vTstep) &
              + lvWS%lvf_ROreclaimed &
              + lvWS%lvf_Reclaimed)
             !
             lvWB%lv_fluxFromWS_Potable(i,vTstep)=lvWS%lvf_ROreclaimed
             lvWB%lv_fluxFromWS_nonPotable(i,vTstep)=lvWS%lvf_Reclaimed
             lvWB%lv_fluxFromWS_Groundwater(i,vTstep)= lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep) &
              + lvWB%lvf_fluxGWaterToWSupply(i,vTstep)
             !
             ! -------------------------------------------------------------------------------
             if(gvl_maskCAP(i))then
              lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep)=lvWB%lvf_fluxFromSWTP_WS_acft(i,vTstep) &
               -  lvWB%lvf_fluxWStoDirectInjection(i,vTstep)
             endif
            !
            lvWB%lv_fluxFromWS_Res_nonPotable(i,vTstep) =anint(P%Out_1*lvWB%lv_fluxFromWS_nonPotable(i,vTstep))
             lvWB%lv_fluxFromWS_Res_ROPotable(i,vTstep)   =anint(P%Out_1*lvWB%lv_fluxFromWS_Potable(i,vTstep))
              lvWB%lv_fluxFromWS_Res_Groundwater(i,vTstep) =anint(P%Out_1*lvWB%lv_fluxFromWS_Groundwater(i,vTstep))
               lvWB%lv_fluxFromWS_Res_SRPandCAP(i,vTstep)   =anint(P%Out_1*lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep))
            !
            lvWB%lv_fluxFromWS_Res_Potable(i,vTstep)    = lvWB%lv_fluxFromWS_Res_ROPotable(i,vTstep) &
              +  lvWB%lv_fluxFromWS_Res_Groundwater(i,vTstep) +  lvWB%lv_fluxFromWS_Res_SRPandCAP(i,vTstep)
            ! -----------------------------------------
            lvWB%lv_fluxFromWS_CIO_nonPotable(i,vTstep) =anint(P%Out_2*lvWB%lv_fluxFromWS_nonPotable(i,vTstep))
             lvWB%lv_fluxFromWS_CIO_ROPotable(i,vTstep)  =anint(P%Out_2*lvWB%lv_fluxFromWS_Potable(i,vTstep))
              lvWB%lv_fluxFromWS_CIO_Groundwater(i,vTstep)=anint(P%Out_2*lvWB%lv_fluxFromWS_Groundwater(i,vTstep))
               lvWB%lv_fluxFromWS_CIO_SRPandCAP(i,vTstep)  =anint(P%Out_2*lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep))
            !
            lvWB%lv_fluxFromWS_CIO_Potable(i,vTstep)    = lvWB%lv_fluxFromWS_Cio_ROPotable(i,vTstep) &
              +  lvWB%lv_fluxFromWS_CIO_Groundwater(i,vTstep) +  lvWB%lv_fluxFromWS_CIO_SRPandCAP(i,vTstep)

            ! ----------------------------------------------
              ! changed from 12 to 13 on 08.30.11 - changed back
              lvWB%lvf_fluxDemandDeficit(i,vTstep)= gvf_WaterDemand_AF(i,12,1)
              !
            ! ----------------------------------------------
if(gvl_mask(i))then
endif

            ! =================== 
            vState_WB_WaterSupply(T%year,i,vTstep)=nint(vState_WB_WaterSupply(T%year,i,vTstep) &
                + lv_fluxToWS(i,vTstep) &
                - (lvWB%lv_fluxFromWS_SRPandCAP(i,vTstep) &
                + lvWB%lv_fluxFromWS_potable(i,vTstep) &
                + lvWB%lv_fluxFromWS_nonPotable(i,vTstep) &
                + lvWB%lvf_fluxWStoDirectInjection(i,vTstep) &
                + lvWB%lv_fluxFromWS_Groundwater(i,vTstep))) 
            !
            ! ---------------------
            ! Update states
            ! -----------------
            !
             call sUpdateState(T,i,vTstep,vState_WB_WaterSupply)
            !
         return
        end subroutine sWSupply
        ! ---------------------
  
        ! -------------------------------
        subroutine sWWTP(T,lvWB,i,vTstep)
        
            integer :: i
            integer :: vTstep    
            integer,parameter :: state=12
            !
            real :: lv_fluxWWinRaw(gvi_maxProV,gpi_timeStepWB)

            type (waterbalance)lvWB     
            type(parameters)P
            type(runTime)T
            ! ===============
              !
                P%In_1=gvf_parm_WWtoEffluent(i)
                P%In_2=gvf_parm_WWtoIndReuse(i)
                P%In_3=0
                P%In_4=0
               call sProportion(state,i,vTstep,parms_WWTP,initParms,P)
                gvf_parm_WWtoEffluent(i)=P%Out_1
                gvf_parm_WWtoIndReuse(i)=P%Out_2
               !
               ! -------------------------------
               !
              ! called "Raw" befcause we have looped WW from FSP
              lv_fluxWWinRaw(i,vTstep)=0
              lv_fluxWWinRaw(i,vTstep)=(1.-gvf_parm_WWtoRWWTP(i))*(lvWB%lv_fluxResidentialWW(i,vTstep) &
                 +lvWB%lv_fluxCioWW(i,vTstep))
              !
              ! -------------------------------
             !
             lvWB%lv_fluxWWTPToEffluent(i,vTstep)=P%Out_1*lv_fluxWWinRaw(i,vTstep)

            ! Flux to industrial re-use ---- check this formulation
             lvWB%lv_fluxWWToIRuse(i,vTstep)=P%Out_2 * lvWB%lv_fluxCioWW(i,vTstep)

             lvWB%lv_fluxWWToSurfaceDischarge(i,vTstep)=lv_fluxWWinRaw(i,vTstep) &
                - ( lvWB%lv_fluxWWTPToEffluent(i,vTstep)+lvWB%lv_fluxWWToIRuse(i,vTstep) )

               ! -------------------------------------------------------
              !
              !
            ! ============== 
            vState_WB_WWTP(T%year,i,vTstep)= vState_WB_WWTP(T%year,i,vTstep) &
                + ( lv_fluxWWinRaw(i,vTstep))&
                - (lvWB%lv_fluxWWTPToEffluent(i,vTstep)+ lvWB%lv_fluxWWToSurfaceDischarge(i,vTstep)  &
                    + lvWB%lv_fluxWWToIRuse(i,vTstep) ) &
                + lvWB%lv_fluxToWWTPFromIRuse(i,vTstep)
             !
            ! ---------------------
            ! Update states
            ! -----------------
            ! 
             call sUpdateState(T,i,vTstep, vState_WB_WWTP)
            !
          return
         end subroutine sWWTP
        ! -------------------
        ! ===========================================================================================
        !
        !  Special states
        ! ---------------


       ! Agriculture
       ! ---------------------------------------
        subroutine sAgriculture(T,lvWB,i,vTstep)

            integer :: i
            integer :: vTstep
            integer,parameter :: state=21
            real,parameter :: lpf_AgEvap=0.793
            real,parameter :: lpf_AgToVadose=0.15
            real :: lvf_AgNet,lvf_AgEvap,lvf_AgToVadose
            real :: lvf_Ag_In
            real :: vArea(gvi_maxProV),lvf_RelativeArea(gvi_maxProV),lvf_TotalArea ! m2
            logical :: lvl_mask(gvi_maxProV)

            type (waterbalance)lvWB
            type(runTime)T
           ! ===============
            !
            if(Citi_newYear)then
              call sMaskAgriculture(gvl_mask)
              gvf_AgNet=0.
            endif
            ! Ag groundwater pumped
            !   This is redundant code because Ag must come before Vadose, yet Aquifer is last
            ! So, repeated in Aquifer subroutine. At present, read in from file, so no issue
            ! 07.05.11 das
            ! ------------------------------------------------------------------------------
              !
             call ProviderArea2012i(i,lvf_RelativeArea)

!              lvf_RelativeArea(i)=vArea(i)/lvf_TotalArea
              !
             lvf_Ag_In=0
            if(i == 1)gvf_AgIn=gv_AgWaterPumpingSRV(T%year,1)
             lvf_Ag_In=gvf_AgIn+lvWB%lvf_fluxEffluentToAg(i,vTstep)
            !
             ! 03.05.12
              ! We are adding the possibility of using ag water to Municipal and Commercial/ Industrial
              ! This is a simple, first cut for the transition to add a more robust treatment of
              ! Agricultural water into the model.
                !
                lvf_Ag_In= max(0,lvf_Ag_In-gvi_WaterFromAgPumping_AF_a(i))
                gvf_AgIn=max(0,gvf_AgIn-gvi_WaterFromAgPumping_AF_a(i))
                !
              !
            lvWB%lv_fluxGWtoAgIrrigation(i,vTstep)= lvf_Ag_In
            !
             lvf_AgEvap=0
            lvf_AgEvap=lpf_AgEvap*(lvf_Ag_In)
            !
                lvf_AgToVadose=0
                lvf_AgToVadose=lpf_AgToVadose*(lvf_Ag_In)
                !
            lvWB%lv_fluxToVadoseFromAg(i,vTstep)=0
            if(gvl_mask(i))then
                 !  NOT all water providers will have Ag, so only those
                ! that do have vadose flux

              lvWB%lv_fluxToVadoseFromAg(i,vTstep)=lvf_RelativeArea(i) &
                * lvf_AgToVadose

            endif
            ! 
            gvf_AgNet=gvf_AgNet+lvWB%lvf_fluxEffluentToAg(i,vTstep) &
              - lvf_AgEvap - lvf_AgToVadose
            !
            gvf_AG_Banking(T%year)= gvf_AG_Banking(T%year)+gvf_AgNet
            !
         return
        end subroutine sAgriculture
        ! -------------------------

        ! Power plant
        ! ------------------------------------
        subroutine sPowerPlant(T,lvWB,i,vTstep)

            integer :: i
            integer :: vTstep
            real :: lvf_evaporation(gpi_timeStepWB)
            real :: vK9=0.79

            type (waterbalance)lvWB
            type(runTime)T
           ! ===============
            ! 
            if(Citi_newYear)then
              gvf_EffluentPP=0
            endif
            ! Includes some effluent from Agua Fria via Avondale and El Mirage (no peoria?)
            ! NEED to parameterize the evaporation rate 05.25.11 das
            lvf_evaporation(vTstep)=lvWB%lv_fluxWWtoPaleoVerde(i,vTstep)*vK9
            gvf_EffluentPP=gvf_EffluentPP+lvWB%lv_fluxWWtoPaleoVerde(i,vTstep)- lvf_evaporation(vTstep)
            !
          return
         end subroutine sPowerPlant
        ! ------------------------

        ! Utilities
        ! -------------------------------------------
        subroutine sBalanceWS(T,lvWB,i,vTstep,lvWS)
            integer :: i
            integer :: vTstep
            real :: lvf_indoorDemand,lvf_Reclaimed,lvf_ROreclaimed
            real :: lvf_BWaterUsed,lvf_BWaterNotUsed,lvf_indoor_deficit

            type typeWS
             real :: lvf_Reclaimed,lvf_ROreclaimed
            end type typeWS
            type(typeWS) lvWS
            type (waterbalance)lvWB
            type(runTime)T
           ! ===============
            ! 
              lvf_indoorDemand=0
              lvf_indoorDemand=  lvWB%lvf_demand_Res_indoor_AF_(i,vTstep) &          
                + lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep) 
              !
               lvf_Reclaimed=0
              if(2000 < T%year)then
                lvf_Reclaimed=gvf_reclaimedOutput(T%year-1,i,vTstep)
              else
                lvf_Reclaimed=gvf_reclaimedOutput(T%year,i,vTstep)
              endif

               lvf_ROreclaimed=0
              if(2000 < T%year)then
                lvf_ROreclaimed=gvf_ROreclaimedOutput(T%year-1,i,vTstep)
              else
                lvf_ROreclaimed=gvf_ROreclaimedOutput(T%year,i,vTstep)
              endif
              !
               call sBlackWater(lvWB,i,lvf_indoorDemand,lvf_Reclaimed, &
                  lvf_BWaterUsed,lvf_BWaterNotUsed,lvf_indoor_deficit)
              !
               gvf_ROreclaimedInput(T%year,i,vTstep)=0
              ! Not all demand have been met
              if(0 < gvf_WaterDemand_AF(i,12,1))then
                 gvf_reclaimedBalance(T%year,i,vTstep)= gvf_reclaimedBalance(T%year,i,vTstep) &
                    + lvf_BWaterNotUsed
                !
                 gvf_ROreclaimedInput(T%year,i,vTstep)= lvf_ROreclaimed
                !
              else
                  ! Both defined in kernel loop ()
                 gvf_ROreclaimedInput(T%year,i,vTstep)= lvf_ROreclaimed
              endif
              !
              if(2000 < T%year)then
                lvWB%lvf_fluxReclaimedDifference(i,vTstep)=  gvf_reclaimedOutput(T%year-1,i,vTstep)-gvf_reclaimedInput(T%year,i,vTstep)
              else
                lvWB%lvf_fluxReclaimedDifference(i,vTstep)=  gvf_reclaimedOutput(T%year,i,vTstep)-gvf_reclaimedInput(T%year,i,vTstep)
              endif
              lvWS%lvf_Reclaimed=gvf_reclaimedInput(T%year,i,vTstep)
              lvWS%lvf_ROreclaimed=gvf_ROreclaimedInput(T%year,i,vTstep)
              !
          return
        end subroutine sBalanceWS
        !
       subroutine sProportion(state,i,j,count,init,P)
          integer :: i,j
          integer :: count,state
          logical :: init
          ! As of this writing, 25 "state variables" in the City Model
          real :: Last_1(gvi_maxProV,25),Last_2(gvi_maxProV,25),Last_3(gvi_maxProV,25),Last_4(gvi_maxProV,25)
          real :: sum,sum2,diff,mod
          real :: lvf_init_1(gvi_maxProV,25),lvf_init_2(gvi_maxProV,25),lvf_init_3(gvi_maxProV,25),lvf_init_4(gvi_maxProV,25)
          real :: check_sum

          type(parameters)P
          !
            !
            ! If no value is assigned to Last_n()i (T=0)then a value must be assigned
            ! -----------------------------------------------------------------------
            ! Start of a new year, zero out
!            !
             if(init)then
              Last_1(i,state)=0
              Last_2(i,state)=0
              Last_3(i,state)=0
              Last_4(i,state)=0
             endif
             sum2=0
            sum2=Last_1(i,state)+Last_2(i,state)+Last_3(i,state)+Last_4(i,state)
              !
              P%Out_1=0
              P%Out_2=0
              P%Out_3=0
              P%Out_4=0
            if(0 < sum2)then
            else
              mod=0
             if(sum2 < 2)then
                sum2=0
               sum2=P%In_1+P%In_2+P%In_3+P%In_4
              if(1 < sum2)then
               if(count == 4)mod=(sum2-1) * 0.25
               if(count == 3)mod=(sum2-1) * (1./3)
               if(count == 2)mod=(sum2-1) * 0.5
               if(count == 1)mod=(sum2-1)
              else
              endif
             else

             endif
              P%In_1=max(0,P%In_1-mod)
              P%In_2=max(0,P%In_2-mod)
              P%In_3=max(0,P%In_3-mod)
              P%In_4=max(0,P%In_4-mod)
            endif
          !
            sum=0
           sum=P%In_1+P%In_2+P%In_3+P%In_4
          !
          if(0 < sum .and. sum <= 1)then
             !
              if(init)then
                if(j <2)then
                 lvf_init_1(i,state)= P%In_1
                 lvf_init_2(i,state)= P%In_2
                 lvf_init_3(i,state)= P%In_3
                 lvf_init_4(i,state)= P%In_4
                endif
              endif
             !
             Last_1(i,state)=P%In_1 * (1./sum)
             Last_2(i,state)=P%In_2 * (1./sum)
             Last_3(i,state)=P%In_3 * (1./sum)
             Last_4(i,state)=P%In_4 * (1./sum)
            !
            P%Out_1=P%In_1
            P%Out_2=P%In_2
            P%Out_3=P%In_3
            P%Out_4=P%In_4
          else
              if(sum < 2)then
               diff=sum-1.
              else
               ! use  initial values (for the year)
               ! when no other ara avail
               ! --------------------
               P%Out_1= lvf_init_1(i,state)
               P%Out_2= lvf_init_2(i,state)
               P%Out_3= lvf_init_3(i,state)
               P%Out_4= lvf_init_4(i,state)
               !
              endif
               ! Create a proportional reduction in each parameter estimate 
               ! ----------------------------------------------------------
               P%Out_1=max(0,P%In_1-(Last_1(i,state)*diff))
               P%Out_2=max(0,P%In_2-(Last_2(i,state)*diff))
               P%Out_3=max(0,P%In_3-(Last_3(i,state)*diff))
               P%Out_4=max(0,P%In_4-(Last_4(i,state)*diff))
          endif
          !
          check_sum=P%Out_1+P%Out_2+P%Out_3+P%Out_4
          if(1 < check_sum)goto 10
          !
          return
10      continue
          !
          if(gvl_WriteToDisk)then
      write(7,*)"State= ",state,"Prov= ",i,"Time step=",j," Check sum > 1 in CitiModel"
          endif
          stop
          !      
        end subroutine sProportion
        ! -------------------------

          ! ----------------------------------------------------------
            subroutine sRangeCheck(state,i,j,minValue,maxValue,in,out)
              integer :: state
              integer :: i,j
              real :: minValue,maxValue
              real :: in,out
                !
                out=min(maxValue,max(minValue,in))
                if(out /= in)then
                   if(gvl_WriteToDisk)write(7,*)"Boundary exceeded: ", state," provider= ",i,"Time step= ",j
                endif
                !
             return
            end subroutine sRangeCheck
            ! -------------------------

          ! --------------------------------------------------------------------
          subroutine sFluxToAtmos(lvWB,i,vTstep,vK1,vK2,lvf_outdoorUse,lvf_flux)
            integer :: i,vTstep
            real :: vK1,vK2
            real :: lvf_flux,lvf_outdoorUse

            type (waterbalance)lvWB          
            ! =====================
            !

           ! ----------------------------------
               lvf_flux=0
              if((1.-vK1) < vK2)then
                lvf_flux=nint((1-vK1)*lvf_outdoorUse)
              else if((1.-vK1) > vK2)then
                lvf_flux=nint((1-vK1)*lvf_outdoorUse)
              else
                lvf_flux=nint(vK2*lvf_outdoorUse)
              endif
             !
              if(0 < lvf_flux)then
               lvWB%lv_fluxAtmosTotal(i,vTstep)=lvWB%lv_fluxAtmosTotal(i,vTstep)+lvf_flux
              endif
             ! --------------------------------
          return
          end subroutine sFluxToAtmos
          ! -------------------------

          ! ----------------------------------------------
            subroutine sUpdateState(T,i,vTstep,stateArray)
              integer :: i,vTstep

              real :: state
              real :: stateArray(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)

              type(runTime)T
              ! ===============
              !
               state=0
              state=max(0,anint(stateArray(T%year,i,vTstep)))
              if(gpi_timeStepWB /=1)then
                if(vTstep == 12)then
                  stateArray(T%year+1,i,1)=state
                else
                  stateArray(T%year,i,vTstep+1)=state
                endif
              else
                stateArray(T%year+1,i,vTstep)=state
              endif
              !
             return
            end subroutine sUpdateState
          ! ---------------------------

          ! --------------------------------------
          subroutine sStepUpdate(T,i,vTstep,state)
              integer :: i,vTstep
              integer :: state
              type(runTime)T
              ! ===============
              !
              if(gpi_timeStepWB /=1)then
                if(vTstep == 12)then
                  if(state == 2)then
!                   gvf_GW_Banking(T%year+1,i,1)=gvf_GW_Banking(T%year,i,vTstep)
                  else if(state == 14)then
                   gvf_ROreclaimedInput(T%year+1,i,1)= gvf_ROreclaimedInput(T%year,i,vTstep)
                  else if(state == 15)then
                   gvf_reclaimedOutput(T%year+1,i,1)= gvf_reclaimedOutput(T%year,i,vTstep)
                  endif
                else
                  if(state == 2)then
!                   gvf_GW_Banking(T%year,i,vTstep+1)=gvf_GW_Banking(T%year,i,vTstep)
                  else if(state == 14)then
                   gvf_ROreclaimedInput(T%year,i,vTstep+1)= gvf_ROreclaimedInput(T%year,i,vTstep)
                  else if(state == 15)then
!                   gvf_reclaimedInput(T%year,i,vTstep+1)= gvf_reclaimedInput(T%year,i,vTstep)
                  endif
                endif
              else
                  if(state == 2)then
!                   gvf_GW_Banking(T%year+1,i,1)=gvf_GW_Banking(T%year,i,vTstep)
                  else if(state == 14)then
                   gvf_ROreclaimedOutput(T%year+1,i,1)= gvf_ROreclaimedOutput(T%year,i,vTstep)
                   ! 07.28.11
                   gvf_ROreclaimedInput(T%year+1,i,1)= gvf_ROreclaimedOutput(T%year,i,vTstep)

                  else if(state == 15)then
                   gvf_reclaimedOutput(T%year+1,i,1)= gvf_reclaimedOutput(T%year,i,vTstep)
                  endif
              endif
              !
            return
          end subroutine sStepUpdate
          ! ------------------------

          ! ----------------------------------------------------------------------------------------------
          subroutine sGrayWater(T,lvWB,i,vTstep,lvf_indoorWater,lvf_tWater,lvf_BW,lv_flux,lv_providerGPCD)
            integer :: i,vTstep
            real :: lvf_indoorWater,lvf_tWater,lvf_BW
            real :: lv_GWavailable
            integer :: lv_flux(gpi_lBY:gpi_uBY,gvi_maxProV,gpi_timeStepWB)
            integer :: lv_fluxReclaimedGreywater
            real ::  lv_providerGPCD(gvi_maxProV,2)

            type (waterbalance)lvWB      
            type(runTime) T 
            ! =====================
            !
              lv_GWavailable=0
             lv_GWavailable=max(0,lvf_indoorWater-lvf_BW)

             lv_fluxReclaimedGreywater=nint(lv_GWavailable)
              !
              lv_providerGPCD(i,1)=(lvf_tWater*gpd_galperacft)*(1/lvWB%lv_providerPop(i)) *(1/gpf_daysMonth)
              lv_providerGPCD(i,2)=(lvf_tWater-lv_GWavailable)*gpd_galperacft &
                 *(1/lvWB%lv_providerPop(i))* (1/gpf_daysMonth)
              !
               !
              if(1 < gpi_timeStepWB)then
               if(vTstep == 12)then
                lv_flux(T%year+1,i,1)= lv_fluxReclaimedGreywater
               else
                lv_flux(T%year,i,vTstep+1)= lv_fluxReclaimedGreywater
               endif
              else
                lv_flux(T%year+1,i,vTstep)= lv_fluxReclaimedGreywater
              endif
             !
            return
          end subroutine sGrayWater
          ! -----------------------

         ! --------------------------------------------------------------------------------------------
          subroutine sBlackWater(lvWB,i,lvf_demand,lvf_nonpotable,lvf_BWused,lvf_BWnotUsed,lvf_deficit)
            integer :: i
            real :: lvf_BWused,lvf_BWnotUsed
            real :: lvf_deficit,lvf_nonpotable
            real :: lvf_demand

            type (waterbalance)lvWB          
            ! =====================
            !
            lvf_BWtypical=0
             if(gvl_parm_toiletPct)then
               lv_gallonsPerFlush=3.63 !  page 95- Mayer and DeOreo (1999)
              if(0 < gvf_parm_LitersPFres(i))lv_gallonsPerFlush=gvf_parm_LitersPFres(i)*1/gpf_gallonsToliters

               lv_flushesPerDay=5.05
              if(0 < gvf_parm_FlushPDres(i))lv_flushesPerDay=gvf_parm_FlushPDres(i)
              lvf_BWtypical= lvWB%lv_providerPop(i)*(lv_gallonsPerFlush*lv_flushesPerDay*gpf_daysMonth)*1/gpd_galperacft
             else
               lvf_BWpct=0.26696 ! page 88- Mayer and DeOreo (1999)
              if(0 < gvf_parm_BWpctRes(i))lvf_BWpct=gvf_parm_BWpctRes(i)
              lvf_BWtypical= (1.-gv_RateResLeak(i))*lvf_demand*lvf_BWpct
              !
             endif
             ! --------------------------------------
              !
               lvf_BWused=0 ; lvf_BWnotUsed=0; lvf_deficit=0
              if(0 < lvf_nonpotable)then
                if(lvf_BWtypical <= lvf_nonpotable)then
                  lvf_BWused=lvf_BWtypical
                  lvf_BWnotUsed=lvf_nonpotable-lvf_BWaterUsed
                else
                  lvf_BWused=lvf_nonpotable + (lvf_BWtypical-lvf_nonpotable)
                  lvf_deficit=(lvf_BWtypical-lvf_nonpotable)
                endif
              else
                  lvf_BWused=lvf_BWtypical
              endif
              !
           return
          end subroutine sBlackWater
          ! ------------------------



        ! -----------------------------------------------------------------------------------------

        ! Surface water subroutines from rainfall
        ! ------------------------------------------ 
        subroutine ProviderRCN(Provider,lvf_xbarRCN)
            integer :: Provider,i
            real :: lvf_RCN(12) ! m2
            real :: lvf_PctLandCover(12)
            real :: lvf_add,lvf_totalRCN
            real :: lvf_xbarRCN(gvi_maxProV) ! individual provider-specific possible, though not yet used
            !
            !http://onlinemanuals.txdot.gov/txdotmanuals/hyd/nrcs_runoff_curve_number_methods.htm
            lvf_RCN(1)=61 ! Grass 39,61,74,80  correspond to A,B,C,D soil groups (see pub)
            lvf_RCN(2)=98 ! Paved all (A,B,C,D) =98
            lvf_RCN(3)=85 ! Gravel 76,85,89,91
            lvf_RCN(4)=82 ! "Dirt" 72,82,87,89 
            lvf_RCN(5)=77 ! Desert Landscaping 63,77,85,88
            lvf_RCN(6)=96 ! Artificial Desert 96
            lvf_RCN(7)=92 ! Commercial 89,92,94,95
            lvf_RCN(8)=88 ! Industrial 81,88,91,93
            lvf_RCN(9)=70 ! Residential -mesic-(using 1/2 acre lot) 54,70,80,85
            lvf_RCN(10)=85 ! Residedntial -xerix- (using 1/8 acre lot size) 77,85,90,92
            lvf_RCN(11)=83 ! "cultivated" 74,83,88,90
            !
            lvf_PctLandCover(1)=1 ! Cultivated Grass
            lvf_PctLandCover(2)=12 ! Asphalt
            lvf_PctLandCover(3)=2  ! River Gravels
            lvf_PctLandCover(4)=7  ! compacted soil (prior ag use and other)
            lvf_PctLandCover(5)=16 ! Undisturbed
            lvf_PctLandCover(6)=1  ! Not sure on this
            lvf_PctLandCover(7)=6  ! assuming commercial
            lvf_PctLandCover(8)=6  ! assuming industrial
            lvf_PctLandCover(9)=14 ! residential
            lvf_PctLandCover(10)=31 ! residential
            lvf_PctLandCover(11)=4  ! "vegetation"
            !
            lvf_totalRCN=0.
            lvf_add=0.
            do i = 1,11,1
             lvf_add=(lvf_RCN(i)*(lvf_PctLandCover(i)/100))
             lvf_totalRCN=lvf_totalRCN+lvf_add
               lvf_xbarRCN(Provider)=lvf_totalRCN
            end do
            !
         return
        end subroutine ProviderRCN
        ! ------------------------

        ! -----------------------------------------------------
        subroutine sRunoff(RCN,lvf_rainfall,fRunoff,lvf_Storage)
          real :: lvf_rainfall
          real :: fRunoff
          real :: lvf_Storage,RCN
          real :: lvc_z=254
            !
            ! rainfall in mm
            !
             fRunoff=0.
             lvf_Storage=0.
            lvf_Storage = lvc_z *(100 * (1./RCN) -1.)
            if(0 < lvf_rainfall) &
             fRunoff = (lvf_rainfall-0.2*lvf_Storage)**2  * (1./ (lvf_rainfall + 0.8*lvf_Storage))
            !
          return
        end subroutine sRunoff
        ! 
        ! --------------------------------------------
        !
        function fPercolation(lvf_rainfall,lvf_Storage)
        
          real :: lvf_rainfall
          real :: fPercolation
          real :: lvf_Storage
          real, parameter :: lvc_scale=3,lvc_store=0.05,lvc_slope=0.1
            !
            ! rainfall in mm, lvf_Storage in mm
            !
             fPercolation=0.
            if(0 < lvf_rainfall) &
             fPercolation = lvc_scale * (lvf_rainfall / (lvc_store*lvf_Storage + lvc_slope*lvf_rainfall ))
            !
        end function fPercolation
        ! 
        ! -----------------------------------------
        !
        subroutine sRainfallRunoff(T,lvWB,i,vTstep)

           integer :: i,j,k
           integer :: vTstep
           real :: lv_RunoffPerc(gvi_maxProV,gpi_timeStepWB)
           real :: lv_fluxToAtmosRain(gvi_maxProV,gpi_timeStepWB)
           real :: vArea(gvi_maxProV) ! m2

           real :: lv_annualRain(gvi_maxProV,gpi_timeStepWB),lv_monthlyRain(gvi_maxProV,gpi_timeStepWB)
           real :: lv_mmTom=0.001
           real :: vR1=0.05                                 ! proportion of runoff leaving cities
           !real :: vR2=0.5                                  ! proportion of runoff evaporating
           real :: cvf_vR3=0.207    ! This needs checking
           real :: lvf_convertRain(gvi_maxProV,gpi_timeStepWB)
           real :: lvf_newRunoff(gvi_maxProV,gpi_timeStepWB),lvf_xbarRCN(gvi_maxProV),lvf_perc(gvi_maxProV,gpi_timeStepWB),lvf_S_mm,lvf_runoff_mm
           logical :: lvl_NCEPmetData

           type (waterbalance)lvWB
!           type(Provider)vPV
           type(runTime)T
           ! ===============
             !
              lvl_NCEPmetData=.true.
             !
              !
              ! lvf_MetData(i,j,k) array is year,month,met variable;k= 1=Tmax,2=Tmin,3=Precip
              !
              !http://onlinemanuals.txdot.gov/txdotmanuals/hyd/nrcs_runoff_curve_number_methods.htm
              !
              ! Runoff Q = (P-0.2*S)^2 / (P + 0.8*S)
              !
              ! P = precipitation (inches or mm)
              ! S = z * (100/RCN - 1) where z=10 (for inches, 254 for mm)

              ! RCN = Runoff Curve Number
              !
             ! Rainfall array is (year,month,provider)
              do j = 1,gvi_Providers,1
                do k = 1,gpi_timeStepWB,1
                  lv_annualRain(j,k)=0.
                  lv_monthlyRain(j,k)=0.
                end do
              end do
             !
              call ProviderArea2012(vArea)

            if(gpi_timeStepWB /=1)then
              !
              call ProviderRCN(i,lvf_xbarRCN) ! Area-weighted (land cover type) Runoff Curve Number

              if(lvl_NCEPmetData)then
                lv_monthlyRain(i,vTstep)=0
               lv_monthlyRain(i,vTstep)= lvf_MetData(T%year,vTstep,3)
               !
                lvf_convertRain(i,vTstep)=0
                lvf_runoff_mm=0
                if(0 < lv_monthlyRain(i,vTstep))then
                 call sRunoff(lvf_xbarRCN(i),lv_monthlyRain(i,vTstep),lvf_runoff_mm,lvf_S_mm)
                 lvf_convertRain(i,vTstep)=vArea(i)* lv_monthlyRain(i,vTstep)*lv_mmTom*gpd_m3Toacft
                endif

                 lvf_perc(i,vTstep)=0
                lvf_perc(i,vTstep)=cvf_vR3*lvf_convertRain(i,vTstep)
                lvf_perc(i,vTstep)=fPercolation(lv_monthlyRain(i,vTstep),lvf_S_mm)
                lvWB%lv_runoffPerc(i,vTstep)=lvf_perc(i,vTstep)

                 lvf_newRunoff(i,vTstep)=0
                lvf_newRunoff(i,vTstep)=(vArea(i)*lvf_runoff_mm*lv_mmTom*gpd_m3Toacft)
                lvWB%lvf_runoff(i,vTstep)=lvf_newRunoff(i,vTstep)
                !
                 lv_fluxToAtmosRain(i,vTstep)=0.
                if((lvf_newRunoff(i,vTstep)+lvf_perc(i,vTstep))< lvf_convertRain(i,vTstep))then
                 lv_fluxToAtmosRain(i,vTstep)= lvf_convertRain(i,vTstep)-lvf_newRunoff(i,vTstep)-lvf_perc(i,vTstep)
                endif
                lvWB%lvf_rainfallEvaporation(i,vTstep)= lv_fluxToAtmosRain(i,vTstep)
                !
              else
!                lv_monthlyRain(i,vTstep)=lv_rainfall(T%year,vTstep,i)
                lv_monthlyRain(i,vTstep)=0
                !
                lv_RunoffPerc(i,vTstep)=(cvf_vR3)*(1.-vR1)*(vArea(i)*lv_monthlyRain(i,vTstep)*lv_mmTom*gpd_m3Toacft)
                lvWB%lv_runoffPerc(i,vTstep)=lv_RunoffPerc(i,vTstep)
                !
                lvWB%lvf_runoff(i,vTstep)=vR1*(vArea(i)*lv_monthlyRain(i,vTstep)*lv_mmTom*gpd_m3Toacft)

                lv_fluxToAtmosRain(i,vTstep)= (1.-cvf_vR3)*(1.-vR1)*(vArea(i)*lv_monthlyRain(i,vTstep)*lv_mmTom*gpd_m3Toacft)
                lvWB%lvf_rainfallEvaporation(i,vTstep)=lv_fluxToAtmosRain(i,vTstep)
                !
              endif
                !
13            format(I4,1x,I2,1x,I2,1x,4(F10.3,1x))
                !
             else
              !
              !
              lv_annualRain(i,1)=0
              do j = 1,12,1
               lv_annualRain(i,1)= lv_annualRain(i,1)+ lvf_MetData(T%year,j,3)
              end do

                lvf_convertRain(i,vTstep)=0
                lvf_runoff_mm=0
                !
                call ProviderRCN(i,lvf_xbarRCN) ! Area-weighted (land cover type) Runoff Curve Number
                !
               call sRunoff(lvf_xbarRCN(i),lv_annualRain(i,1),lvf_runoff_mm,lvf_S_mm)
                 lvf_convertRain(i,vTstep)=vArea(i)* lv_annualRain(i,1)*lv_mmTom*gpd_m3Toacft

                lvf_perc(i,vTstep)=0
               lvf_perc(i,vTstep)=cvf_vR3*lvf_convertRain(i,vTstep)
               lvf_perc(i,vTstep)=fPercolation(lv_annualRain(i,1),lvf_S_mm)

               lvWB%lv_runoffPerc(i,vTstep)=lvf_perc(i,vTstep)

                lvf_newRunoff(i,vTstep)=0
               lvf_newRunoff(i,vTstep)=(vArea(i)*lvf_runoff_mm*lv_mmTom*gpd_m3Toacft)
               lvWB%lvf_runoff(i,vTstep)=lvf_newRunoff(i,vTstep)

               lv_fluxToAtmosRain(i,vTstep)=0.
                if((lvf_newRunoff(i,vTstep)+lvf_perc(i,vTstep))< lvf_convertRain(i,vTstep))then
                 lv_fluxToAtmosRain(i,vTstep)= lvf_convertRain(i,vTstep)-lvf_newRunoff(i,vTstep)-lvf_perc(i,vTstep)
                endif
                lvWB%lvf_rainfallEvaporation(i,vTstep)= lv_fluxToAtmosRain(i,vTstep)
             endif
             !
         return
        end subroutine sRainfallRunoff
        ! ----------------------------

        ! --------------------------------------------------------------------------------------------------------------------


        ! ========================================================================================================================
        ! Outputs
        !
         ! -------------------------------------------
        subroutine TypeTransferIN(T,i,vTstep,vPV,lvWB)
         integer :: i
           integer :: vTstep
 
           type(waterbalance)lvWB
           type(Provider)vPV
           type(runTime)T
           ! ====================
            !
              call sMask(gvl_mask)
            !
            lvWB%lv_providerPop(i)=vPV%cPopProvider(i)
            !
            ! Water demand on a monthly basis
            if(gpi_timeStepWB /=1)then
              if(i == 1 .and. vTstep == 1)then
                  !
              endif

               lvWB%lvf_demand_Res_outdoor_AF_(i,vTstep) &
                    = vPV%lvf_cDemand_Res_outdoor_Af_monthly(i,vTstep)
               lvWB%lvf_demand_Res_indoor_AF_(i,vTstep) &
                    = vPV%lvf_cDemand_Res_indoor_Af_monthly(i,vTstep)
               lvWB%lvf_demand_Cio_outdoor_AF_(i,vTstep)&
                    = vPV%lvf_cDemand_Cio_outdoor_Af_monthly(i,vTstep)
               lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep) &
                    = vPV%lvf_cDemand_Cio_indoor_Af_monthly(i,vTstep)
            else
                ! Annual water demand
               lvWB%lvf_demand_Res_outdoor_AF_(i,vTstep)=anint(vPV%lvf_cDemand_Res_outdoor_AF_annual(i))
               lvWB%lvf_demand_Res_indoor_AF_(i,vTstep) =anint(vPV%lvf_cDemand_Res_indoor_AF_annual(i))
               lvWB%lvf_demand_Cio_outdoor_AF_(i,vTstep)=anint(vPV%lvf_cDemand_Cio_outdoor_AF_annual(i))
               lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep) =anint(vPV%lvf_cDemand_Cio_indoor_AF_annual(i))
            endif
              !
              lvWB%lvf_demand_AF(i,vTstep)=lvWB%lvf_demand_Res_outdoor_AF_(i,vTstep) &
                +  lvWB%lvf_demand_Res_indoor_AF_(i,vTstep) &
                +  lvWB%lvf_demand_Cio_outdoor_AF_(i,vTstep) &
                +  lvWB%lvf_demand_Cio_indoor_AF_(i,vTstep)
              !
            if(gpi_timeStepWB /=1)then
                
            else
                lvWB%lvf_fluxSWaterCAP(i)=(vPV%gvf_cCAP_used_AF_a(i))
                lvWB%lvf_fluxSWaterSRP(i)=(vPV%gvf_cSRP_A_used_AF_a(i)+vPV%gvf_cSRP_NCS_used_AF_a(i)+vPV%gvf_cSRP_BCresAndPump_AF_a(i))
                lvWB%lvf_fluxSWaterSRP_NCS(i)=vPV%gvf_cSRP_NCS_used_AF_a(i)
                lvWB%lvf_fluxSWaterSRP_A(i)=vPV%gvf_cSRP_A_used_AF_a(i)
                lvWB%lvf_fluxGWaterSRP(i)=(vPV%gvf_cSRP_BC_used_AF_a(i))  
                lvWB%lvf_fluxBCstorageSRP(i)=vPV%gvf_cSRP_BCresAndPump_AF_a(i)
              !
                !
                lvWB%lvf_municipalPumping_AF_(i)=(vPV%gvf_cMunicipalPumping_AF(i))
                lvWB%lvf_municipalRecharge_AF_(i)=0
                ! not sure on this....
                lvWB%lvf_fluxGWaterToWSupply(i,vTstep)=(vPV%gvf_cMunicipalPumping_AF(i))
            endif
            !
           ! =========
         return
        end subroutine TypeTransferIN
        ! ---------------------------

        ! --------------------------------------------------
        subroutine TypeTransferOUT(T,i,vTstep,lvWB,vEB,vSB)
         integer :: i
           integer :: vTstep
 
           type(waterbalance)lvWB
           type(EnergyWB)vEB
           type(SalinityWB)vSB
           type(runTime)T

           ! ====================
            !
             ! Send to Energy_CityModel.f90
            ! ---------------------------------------------------------------------
!            vEB%gv_wSWtoWTP(i,vTstep)=lvWB%lv_fluxSurfaceWaterToSWTP(i,vTstep)
!            vEB%gv_wWTPtoWS(i,vTstep)=lvWB%lv_fluxFromSWTP(i,vTstep)
!            vEB%gv_wGWtoAg(i,vTstep)=lvWB%lv_fluxGWtoAgIrrigation(i,vTstep)
!            vEB%gv_wGWtoWTP(i,vTstep)=lvWB%lv_fluxGWtoWTP(i,vTstep)
!            vEB%gv_wEffluenttoPower(i,vTstep)=lvWB%lv_fluxWWtoPaleoVerde(i,vTstep)
!            !
            vEB%gv_wSRPtoE(vTstep)=lvWB%lv_SRPtoEnergy(vTstep)
!           !
!           !
!            ! Send to TDS_CityModel.f90
!             !---------------------------------------------------------------------------------------
             vSB%gvf_sFluxSWTPtoWS_SW_acft(i,vTstep)    =lvWB%lvf_fluxFromSWTP_WS_acft(i,vTstep)
!
             vSB%gvf_sFluxWWresToWWTP_acft(i,vTstep)    =lvWB%lv_fluxResidentialWW(i,vTstep)
             vSB%gvf_sFluxWWcioToWWTP_acft(i,vTstep)    =lvWB%lv_fluxCioWW(i,vTstep)
             vSB%gvf_sFluxWWwwtpToRWWTP_acft(i,vTstep)  =lvWB%lv_fluxToRWWTP(i,vTstep)
             vSB%gvf_sFluxWWtwtpToAg_acft(i,vTstep)     =lvWB%lvf_fluxEffluentToAg(i,vTstep)
!
!!             vSB%gvf_sFluxROToDirectInject(i,vTstep)                =lvWB%lv_fluxFromRevOsToVadose(i,vTstep) 
!
             vSB%gvf_sFluxRunoffPerc_acft(i,vTstep)     =lvWB%lv_runoffPerc(i,vTstep)
             vSB%gvf_sGwaterToAgToTDS_acft(i,vTstep)    =lvWB%lv_fluxGWtoAgIrrigation(i,vTstep)
             vSB%gvf_sMuniRechargeToTDS_acft(i,vTstep)  =lvWB%lvf_fluxSWaterToGWaterBanking(i,vTstep)
           !  
             vSB%gvf_sFluxRes_acft(i,vTstep)            =lvWB%lv_fluxResidential(i,vTstep)
              vSB%gvf_sFluxResIndoor_acft(i,VTstep)      =lvWB%lv_fluxResIndoor(i,vTstep)
               vSB%gvf_sFluxResOutdoor_acft(i,vTstep)     =lvWB%lv_fluxResOutdoor(i,vTstep)

            !
             vSB%gvf_sFluxCio_acft(i,vTstep)            =lvWB%lv_fluxCIO(i,vTstep)
              vSB%gvf_sFluxCioIndoor_acft(i,VTstep)      =lvWB%lv_fluxCioIndoor(i,vTstep)
               vSB%gvf_sFluxCioOutdoor_acft(i,vTstep)     =lvWB%lv_fluxCioOutdoor(i,vTstep)

!           ! okay
            vSB%gvf_sCAPtoTDS_acft(i,vTstep)            =lvWB%lvf_fluxSWaterCAP(i)
            vSB%gvf_sSRPclassAtoTDS_acft(i,vTstep)      =lvWB%lvf_fluxSWaterSRP_A(i)
            !
            vSB%gvf_sSRPclassBCtoTDS_acft(i,vTstep)     =lvWB%lvf_fluxGWaterSRP(i)
            vSB%gvf_sSRPclassBCstorage_acft(i,vTstep)   =lvWB%lvf_fluxBCstorageSRP(i)
            vSB%gvf_sSRPNCStoTDS_acft(i,vTstep)         =lvWB%lvf_fluxSWaterSRP_NCS(i)
            !
            vSB%gvf_sFluxGWaterBanking(i,vTstep)        =lvWB%lvf_fluxGWaterBankingToSWater(T%year,i,vTstep)
            vSB%gvf_sReclaimed_acft(i,vTstep)           =gvf_reclaimedInput(T%year,i,1)
            vSB%gvf_sROreclaimed_acft(i,vTstep)         =gvf_ROreclaimedInput(T%year,i,1)

            vSB%gvf_sMuniPumpingtoTDS_acft(i,vTstep)    =lvWB%lvf_municipalPumping_AF_(i)
            vSB%gvf_sMuniPumpingTreatedtoTDS_acft(i,vTstep)=lvWB%lvf_fluxGWaterTreatmentToWSupply(i,vTstep)
            !
         return
        end subroutine TypeTransferOUT
        ! ----------------------------

        ! --------------------------------------------
        subroutine sOutputToInterface(T,lvWB,i,vTstep)
          integer :: i,vTstep
          type(waterbalance)lvWB
          type(runTime)T
          ! ============          
            !
            if(Citi_newYear)then
               goi_EffluentAg_AF=0
              call sMaskSROG(gvl_mask)
            endif
            if(gpi_timeStepWB /=1)then
            else
              ! Avoid rounding errors that could make input > output
              ! 09.01.11 das
              if(gvf_reclaimedOutput(T%year,i,1) < gvf_reclaimedInput(T%year,i,1)) &
                gvf_reclaimedInput(T%year,i,1)=gvf_reclaimedOutput(T%year,i,1)

              goi_GWBanked_AF(i)            =nint(gvf_WBankingBalance(T%year,i,1))
              goi_GWBankUsed_AF(i)          =nint(gvf_GW_Banking(T%year,i,1))

              goi_ReclaimedWtotal_AF(i)     =nint(lvWB%lvf_fluxReclaimedTotal(i,vTstep))
              goi_ReclaimedWoutput_AF(i)    =nint(gvf_reclaimedOutput(T%year,i,1))
              goi_ReclaimedWused_AF(i)      =nint(gvf_reclaimedInput(T%year,i,1))
            
              goi_ReclaimedDifference_AF(i) =nint(lvWB%lvf_fluxReclaimedDifference(i,1))
              goi_ReclaimedWrecharged_AF(i) =lvWB%lvf_fluxReclaimedToVadose(i,1)
              goi_ReclaimedWdischarged_AF(i)=lvWB%lvf_fluxReclaimedToRWWSDischarge(i,vTstep)
              goi_ReclaimedWDI_AF(i)        =nint(lvWB%lvf_fluxReclaimedToDirectInject(i,vTstep))

              goi_ROReclaimedWoutput_AF(i) =nint(gvf_ROreclaimedOutput(T%year,i,1))
              goi_ROReclaimedWused_AF(i)    =nint(gvf_ROreclaimedInput(T%year,i,1))
              goi_ROReclaimedWDI_AF(i)      =nint(lvWB%lv_ROflux_DirectInject_Potable(i,vTstep))

              goi_Effluent_AF(i)            =nint(lvWB%lv_fluxWWTPToEffluent(i,vTstep))
              goi_EffluentPPlant_AF(i)      =nint(lvWB%lvf_fluxEffluentToPowerPlant(i,vTstep))
              goi_EffluentVadose_AF(i)      =nint(lvWB%lv_fluxToVadoseFromEffluent(i,vTstep))
              !
              if(gvl_mask(i))then
                goi_EffluentAg_AF = &
                        goi_EffluentAg_AF + nint( lvWB%lvf_fluxEffluentToAg(i,vTstep))
              endif
              !
            endif
              goi_EffluentToDischarge_AF(i) =nint(lvWB%lv_fluxWWToSurfaceDischarge(i,vTstep))
              goi_DemandDeficit_AF(i)       =nint(lvWB%lvf_fluxDemandDeficit(i,vTstep))

if(gvl_mask(i))then
if(i == 33)then
if(2005 < T%year)then
if(gvl_WriteToDisk)then
!write(105,*)T%year,i, goi_ROReclaimedWcreated_AF(i), goi_ROReclaimedWused_AF(i), goi_ROReclaimedWDI_AF(i)
!write(105,*)goi_GWBanked_AF(i), " banked"
!write(105,*)goi_GWBankUsed_AF(i), " banked used"
!if(0 < goi_DemandDeficit_AF(i))write(104,*)goi_DemandDeficit_AF(i), " deficit"
!write(105,*)gvl_parm_shortage(i), "<shortage ",goi_ReclaimedWcreated_AF(i), " reclaimed created as output"
!write(105,*)goi_ReclaimedWused_AF(i), " reclaimed used"
!write(105,*)goi_ReclaimedWrecharged_AF(i), " reclaimed recharged"
!write(105,*)goi_ReclaimedWdischarged_AF(i), " reclaimed discharged"
!write(105,*)goi_ReclaimedWDI_AF(i), " reclaimed direct injection"
!write(105,*)goi_ROReclaimedWcreated_AF(i), " RO reclaimed created"
!write(105,*)goi_ROReclaimedWused_AF(i), " RO reclaimed used"
!write(105,*)goi_ROReclaimedWDI_AF(i), " RO direct injection"
!write(105,*)goi_Effluent_AF(i), " Effluent"
!write(105,*)goi_EffluentPPlant_AF(i), " Effluent to Power plant"
!write(105,*)goi_EffluentVadose_AF(i), " Effluent to Vadose"
!write(105,*)goi_EffluentAg_AF, " Effluent to Ag"
!write(105,*)goi_EffluentToDischarge_AF(i), " Effluent to Discharge"
!write(105,*)"  "
endif
endif
endif
endif
           
          ! ============
         return
        end subroutine sOutputToInterface
        ! -------------------------------


    ! ================================================================
    ! ------------------------------------------
    subroutine sCityModelWater(T,i,j,vPV,lvWB)
        integer :: i,j
        type(Provider)vPV
        type(waterbalance)lvWB
        type(runTime)T
        ! =====================
          !
             ! ----------------------------------
            if(gvl_IncludeMeteorology)then
              call sRainfallRunoff(T,lvWB,i,j)
            endif
             call sSurfaceWater(T,vPV,lvWB,i,j)
               call sSWTP(T,lvWB,i,j)
                call sWSupply(T,lvWB,i,j)
                 call sResidential(T,lvWB,i,j)
                  call sCommercialIO(T,lvWB,i,j)
                   call sWWTP(T,lvWB,i,j)
                    call sIndustrialReuse(T,lvWB,i,j)
                    call sRWWTP(T,lvWB,i,j)
                   call sROprocess(T,lvWB,i,j)
                  call sReclaimed(T,lvWB,i,j)
                 call sEffluent(T,lvWB,i,j)
                call sDirectInjection(T,lvWB,i,j)
               call sAgriculture(T,lvWB,i,j)
              call sVadose(T,lvWB,i,j)
             call sAquifer(T,vPV,lvWB,i,j)
             !
            call sPowerPlant(T,lvWB,i,j)
            call sOutputToInterface(T,lvWB,i,j)
           ! ----------------------------------
        !
      return
    end subroutine sCityModelWater
    ! ---------------------------
    ! ================================================================
    !
End Module lms_CitiWaterBudgets
!
    ! ----------------
    subroutine sCitiModel(T,vPV,vEO)
      use lms_CitiWaterBudgets
       use gm_ModelControl

         integer :: i,j

         type(Provider)vPV
         type(waterbalance)lvWB
!         type(EnergyWB)vEB
!         type(SalinityWB)vSB
         type(Energy)vEO
         type(runTime)T
        ! ==============
          !
           ! =======================================
              ! -----------------------
               !
               Citi_newYear=.true.
                initParms=.true.
               do i = 1,gvi_Providers,1
                if(i < 2)lvl_firstLoop=.true.
                  !
                  do j= 1,gpi_timeStepWB,1
                    call TypeTransferIN(T,i,j,vPV,lvWB)
                    call sCityModelWater(T,i,j,vPV,lvWB)
                     Citi_newYear=.false.
                     lvl_firstLoop=.false.
!                    call TypeTransferOUT(T,i,j,lvWB,vEB,vSB)
                   end do
                  !
               end do
                 initParms=.false.
                !
!               do i = 1,gvi_Providers,1
!                 do j= 1,gpi_timeStepWB,1
!                  call sCityModelSalinity(T,i,j,vSB)
!                 end do
!               end do
                !
             ! -----------------------
           ! ====================================
        !
      return
    end subroutine sCitiModel
    ! ------------------------
    ! -------------------------------------------------------------------------
!                  call TypeTransferOUT(T,i,j,lvWB,vEB,vSB)
!                  call sCityModelSalinity(T,i,j,vSB)
!                  call sCityModelEnergy(T,i,j,vEB,vEO)

    ! Demand and supply subroutines in Kernel.f90
    ! =========================================================================
    ! ----------------------------------
    subroutine sReclaimed_k(T,gvi_order)
      use lms_CitiWaterBudgets
       use gm_ModelControl

         integer :: i
         integer(1) :: gvi_order
         real :: lvf_reclaimed(gvi_maxProV),lvf_on(gvi_maxProV),lvf_off(gvi_maxProV)
         real :: code=4
         real :: lvf_demand,lvf_propDemand

         type(runTime)T
        ! =====================
          !
        do i = 1,gvi_Providers,1      

           lvf_reclaimed(i)=0
           lvf_demand=0
           lvf_propDemand=0
           lvf_demand=gvf_WaterDemand_AF(i,gvi_order,1)
           !
          if(0 < lvf_demand)then
            if(gpi_timeStepWB /= 1)then
            else
              ! Not sure about this code 07.19.11 - new on 07.26.11
                if(2000 < T%year)then
                    if(0 < gvf_reclaimedOutput(T%year-1,i,1))then
                        if(gvf_reclaimedInputMax(T%year,i) < gvf_reclaimedOutput(T%year-1,i,1))then
                         gvf_reclaimedInput(T%year,i,1)=gvf_reclaimedInputMax(T%year,i)
                        else
                         gvf_reclaimedInput(T%year,i,1)= gvf_reclaimedOutput(T%year-1,i,1)
                        endif                
                    lvf_reclaimed(i)=gvf_reclaimedInput(T%year,i,1)
                    else
                    endif
                else
                   if(0 < gvf_reclaimedOutput(T%year,i,1))then
                        if(gvf_reclaimedInputMax(T%year,i) < gvf_reclaimedOutput(T%year,i,1))then
                         gvf_reclaimedInput(T%year,i,1)=gvf_reclaimedInputMax(T%year,i)
                        else
                         gvf_reclaimedInput(T%year,i,1)= gvf_reclaimedOutput(T%year,i,1)
                        endif                
                    lvf_reclaimed(i)=gvf_reclaimedInput(T%year,i,1)
                    else
                    endif
                endif
              !
            endif
           !
           lvf_on(i)=0
           lvf_off(i)=0
           !
           if(0 < lvf_demand)lvf_propDemand=gvf_WaterDemand_AF(i,gvi_order,3) * (1./lvf_demand)
           lvf_on(i)=lvf_reclaimed(i)*lvf_propDemand
           lvf_off(i)=lvf_reclaimed(i)-lvf_on(i)
          !
          else
           lvf_on(i)=0
           lvf_off(i)=0
          endif
        end do
        !
        call sCWaccounting(gvi_order,lvf_off,lvf_on,code)
        !
        gvi_order=gvi_order+1
        !
      return
    end subroutine sReclaimed_k
    ! -------------------------

   ! ---------------------------------------------
    subroutine sROreclaimed_k(T,gvi_order,lvl_flag)
      use lms_CitiWaterBudgets
       use gm_ModelControl
         integer :: i,j
         integer(1) :: gvi_order
         logical :: lvl_flag(gvi_maxProV)
         real :: lvf_ROreclaimed(gvi_maxProV),lvf_on(gvi_maxProV),lvf_off(gvi_maxProV)
         real :: code=5
         real :: lvf_demand,lvf_propDemand
         real :: lvf_adjustRO=0.65

         type(runTime)T
        ! =====================
          !
          ! ==================================================
          do i = 1,gvi_Providers,1
           lvf_ROreclaimed(i)=0
           lvf_demand=0
           lvf_propDemand=0
           lvf_demand=gvf_WaterDemand_AF(i,gvi_order,1)
           !
           if(0 < lvf_demand)then
            if(gpi_timeStepWB /= 1)then
              do j = 1,12,1
                lvf_ROreclaimed(i)=lvf_ROreclaimed(i)+gvf_ROreclaimedInput(T%year-1,i,j)
              end do
            else
                if(2000 < T%year)then
                    if(0 < gvf_ROreclaimedOutput(T%year-1,i,1))then
                        lvf_ROreclaimed(i)=gvf_ROreclaimedInput(T%year,i,1)
                    endif
                else
                    if(0 < gvf_ROreclaimedOutput(T%year,i,1))then
                        lvf_ROreclaimed(i)=gvf_ROreclaimedInput(T%year,i,1)
                    endif
                endif
            

            endif
            !
            lvf_on(i)=0
            lvf_off(i)=0
            if(0 < lvf_demand)lvf_propDemand=gvf_WaterDemand_AF(i,gvi_order,3) * (1./lvf_demand)
            lvf_on(i)=lvf_ROreclaimed(i)*lvf_propDemand
            lvf_off(i)=lvf_ROreclaimed(i)-lvf_on(i)
            !
           else
            lvf_on(i)=0
            lvf_off(i)=0
           endif
          end do
          !
            call sCWaccounting(gvi_order,lvf_off,lvf_on,code)
          !
          gvi_order=gvi_order+1
          !
    end subroutine sROreclaimed_k
    ! ---------------------------

   ! ------------------------------------------
    subroutine sBanking_k(T,gvi_order,interrupt)
      use lms_CitiWaterBudgets
       use gm_ModelControl
         integer :: i
         integer(1) :: gvi_order
         logical :: interrupt
         real :: lvf_Banking(gvi_maxProV),lvf_on(gvi_maxProV),lvf_off(gvi_maxProV)
         real :: lvf_propDemand
         real :: code=11

         type(runTime)T
         ! =====================
          !
          ! ==================================================
          ! Note: only annual CityModel time step covered here.  Will need to change        
            do i = 1,gvi_Providers,1
              lvf_Banking(i)=0
              gvf_GW_Banking(T%year,i,1)=0
             !
              if(interrupt)then
                gvf_WBankingBalance(T%year,i,1)= gvf_GW_Banking_Hold(i)
              endif
              if(0 < gvf_WBankingBalance(T%year,i,1))then
                if(0 < gvf_WaterDemand_AF(i,gvi_order,1))then
                    if(gvf_WaterDemand_AF(i,gvi_order,1) < gvf_WBankingBalance(T%year,i,1))then
                      lvf_Banking(i)=gvf_WaterDemand_AF(i,gvi_order,1)
                      gvf_WBankingBalance(T%year,i,1)= max(0,gvf_WBankingBalance(T%year,i,1)-lvf_Banking(i))
                    else
                      lvf_Banking(i)=max(0,gvf_WBankingBalance(T%year,i,1))
                      gvf_WBankingBalance(T%year,i,1)=0
                    endif     
                    ! Used in subroutine sSurfaceWater
                    gvf_GW_Banking(T%year,i,1)=lvf_Banking(i)                             
                endif
              !
              else
                lvf_Banking(i)=0
              endif
                !
            end do
            !
            do i = 1,gvi_Providers,1
              lvf_on(i)=0
              lvf_off(i)=0
              lvf_propDemand=0
              !
              if(0 < lvf_Banking(i))then
                lvf_propDemand= gvf_WaterDemand_AF(i,gvi_order,3) * (1./ lvf_Banking(i))
              endif
                  lvf_on(i) =lvf_Banking(i)*lvf_propDemand
                  lvf_off(i)=lvf_Banking(i)-lvf_on(i)
            end do
          !
              call sCWaccounting(gvi_order,lvf_off,lvf_on,code)

          gvi_order=gvi_order+1
          !      
        return
    end subroutine sBanking_k
    ! -----------------------

    ! ----------------------------------------------
    subroutine sDirectInject_s(T,gvi_order,lvl_flag)
      use lms_CitiWaterBudgets
       use gm_ModelControl
         integer :: i
         integer(1) :: gvi_order
         logical :: lvl_flag(gvi_maxProV)

         real :: lvf_DirectInject,lvf_add,lvf_proportion
         real :: code=3

         type(runTime)T
         ! ============
          !          

          ! ======================
          !  This subroutine MUST be called first, as coded- gvf_GW_Banking(T%year,i,vTstep)
          ! -----------------------
          call sMaskCAP(gvl_maskCAP)
          !              
          do i = 1,gvi_Providers,1
            lvf_DirectInject=0
            lvf_proportion=0
            !
            if(gvl_maskCAP(i))then

              if(lvl_flag(i))then

              else
!                lvf_DirectInject=gvf_parm_WStoDI(i)*gvf_parm_WStoDIamount(i)
                lvf_DirectInject=gvf_parm_WStoDIamount(i)

                if(0 < gvf_WaterDemand_AF(i,gvi_order,1))then
                  lvf_proportion= gvf_WaterDemand_AF(i,gvi_order,2) * (1./gvf_WaterDemand_AF(i,gvi_order,1))
                endif
                  lvf_add=0
                  lvf_add= lvf_DirectInject * lvf_proportion
                  !
                gvf_WaterDemand_AF(i,gvi_order,1)=  gvf_WaterDemand_AF(i,gvi_order,1) + lvf_DirectInject            
                if(0 <  gvf_WaterDemand_AF(i,1,3))then
                  gvf_WaterDemand_AF(i,gvi_order,2)= anint(gvf_WaterDemand_AF(i,gvi_order,2)+ lvf_add)
                  gvf_WaterDemand_AF(i,gvi_order,3)= gvf_WaterDemand_AF(i,gvi_order,3)+ anint(lvf_DirectInject -lvf_add)
                else
                  gvf_WaterDemand_AF(i,gvi_order,2)= gvf_WaterDemand_AF(i,gvi_order,2)+ lvf_DirectInject
                  gvf_WaterDemand_AF(i,gvi_order,3)=  gvf_WaterDemand_AF(i,gvi_order,3)+ 0
                endif
              endif
            endif
              !         
              gvf_WaterDemand_AF(i,gvi_order+1,1)= gvf_WaterDemand_AF(i,gvi_order,1)
              gvf_WaterDemand_AF(i,gvi_order+1,2)= gvf_WaterDemand_AF(i,gvi_order,2)
              gvf_WaterDemand_AF(i,gvi_order+1,3)= gvf_WaterDemand_AF(i,gvi_order,3)
              !
              gvf_WaterDemand_AF(i,gvi_order,6)=code
          end do
          !
         gvi_order=gvi_order+1
         !      
      return
    end subroutine sDirectInject_s
    ! ----------------------------

    ! -----------------------------------------
     subroutine sVadose_s(T,gvi_order,lvl_flag)
      use lms_CitiWaterBudgets
       use gm_ModelControl
         integer :: i
         integer(1) :: gvi_order
         logical :: lvl_flag(gvi_maxProV)

         real :: lvf_Vadose(gvi_maxProV),lvf_add,lvf_proportion
         real :: code=2

         type(runTime)T
         ! ============
          !
          call sMaskCAP(gvl_maskCAP)
          !
          ! ======================
          if(gvl_start)then
            do i = 1,gvi_Providers,1
              lvl_flag(i)=.false.
              gvl_parm_shortage(i)=.false.
            end do
          endif
          do i = 1,gvi_Providers,1
            !
            lvf_proportion=0

            !
            if(gvl_maskCAP(i))then
              if(lvl_flag(i))then
                lvf_Vadose(i)=0
                !
              else
                !
                ! Add to the demand so that I can remove later
                ! ---------------------------------
                  lvf_Vadose(i)= gvf_parm_SWtoVadoseAmt(i) 
                  !
                  if(0 < gvf_WaterDemand_AF(i,gvi_order,1))then
                    lvf_proportion= gvf_WaterDemand_AF(i,gvi_order,2)/gvf_WaterDemand_AF(i,gvi_order,1) 
                  endif

                  lvf_add=0
                  lvf_add=lvf_Vadose(i)*lvf_proportion

                  if(0 <  gvf_WaterDemand_AF(i,1,3))then
                    gvf_WaterDemand_AF(i,gvi_order,1)=  gvf_WaterDemand_AF(i,gvi_order,1) + lvf_Vadose(i)
                    gvf_WaterDemand_AF(i,gvi_order,2)=  anint(gvf_WaterDemand_AF(i,gvi_order,2) + lvf_add)
                    gvf_WaterDemand_AF(i,gvi_order,3)=  gvf_WaterDemand_AF(i,gvi_order,3) + anint(lvf_Vadose(i)-lvf_add)
                  else
                    gvf_WaterDemand_AF(i,gvi_order,1)=  gvf_WaterDemand_AF(i,gvi_order,1) + lvf_Vadose(i)
                    gvf_WaterDemand_AF(i,gvi_order,2)=  gvf_WaterDemand_AF(i,gvi_order,2) + lvf_Vadose(i) 
                    gvf_WaterDemand_AF(i,gvi_order,3)=  gvf_WaterDemand_AF(i,gvi_order,3)
                  endif
                endif
              !
              endif

                gvf_WaterDemand_AF(i,gvi_order+1,1)=  gvf_WaterDemand_AF(i,gvi_order,1) 
                gvf_WaterDemand_AF(i,gvi_order+1,2)=  gvf_WaterDemand_AF(i,gvi_order,2)
                gvf_WaterDemand_AF(i,gvi_order+1,3)=  gvf_WaterDemand_AF(i,gvi_order,3) 
              !

                gvf_WaterDemand_AF(i,gvi_order,6)=code
             
            end do
            !
            gvi_order=gvi_order+1
          !
      return
    end subroutine sVadose_s
    ! -----------------------


    ! --------------------------------------------------------
    ! --------
    subroutine sCWaccounting(gvi_order,lvf_in_1,lvf_in_2,code)
     use lms_CitiWaterBudgets

         integer :: i
         integer(1) :: gvi_order
         real :: lvf_in_1(gvi_maxProV),lvf_in_2(gvi_maxProV)
         real :: code
          !
          ! =================================================      
          !  
          do i = 1,gvi_Providers,1
           ! in_1 is "off-project", in_2 is "on-project"
           gvf_WaterDemand_AF(i,gvi_order,4)= anint(max(0,lvf_in_1(i)))
           gvf_WaterDemand_AF(i,gvi_order,5)= anint(max(0,lvf_in_2(i)))
           gvf_WaterDemand_AF(i,gvi_order,6)=code
           !  
              ! -------------------------------------------
               gvf_WaterDemand_AF(i,gvi_order+1,2)=0
              if(0 < gvf_WaterDemand_AF(i,gvi_order,2))then
                   gvf_WaterDemand_AF(i,gvi_order+1,2)= &
                    anint(max(0,(gvf_WaterDemand_AF(i,gvi_order,2)- gvf_WaterDemand_AF(i,gvi_order,4))))
              endif
              !
               gvf_WaterDemand_AF(i,gvi_order+1,3)=0
              if(0 < gvf_WaterDemand_AF(i,gvi_order,3))then
                 gvf_WaterDemand_AF(i,gvi_order+1,3)= &
                  anint(max(0,(gvf_WaterDemand_AF(i,gvi_order,3)- gvf_WaterDemand_AF(i,gvi_order,5))))
              endif
            !
               gvf_WaterDemand_AF(i,gvi_order+1,1)=0
            if(0 < gvf_WaterDemand_AF(i,gvi_order,1))then
             gvf_WaterDemand_AF(i,gvi_order+1,1)= anint( &
                max(0,gvf_WaterDemand_AF(i,gvi_order,1) &
                - (gvf_WaterDemand_AF(i,gvi_order,4) + gvf_WaterDemand_AF(i,gvi_order,5)  ) ))
            endif
            !      
          end do
          !
          ! =============================================================
        return
    end subroutine sCWaccounting
    ! --------------------------
!
!
        ! --------------------------------------------------------
        ! Utilities
        ! ----------
        subroutine sSES(T,startYear,lvf_in,lvf_Reduction_pct,fSES)
          use lms_CitiWaterBudgets
          use gm_ModelControl

          real :: alpha
          real :: lvf_Reduction_pct,lvf_Reduction_proportion 
          integer :: MYyear,startYear
          integer :: k,lvi_referenceYear
          integer :: lvi_diffYear=2080

          real, parameter :: a=81.6 ! This value changed from 80.8 to "correct" drift in proportional estimates
          real, parameter :: b=0.6830
          real, parameter :: c=-0.0113
          real :: lvf_in,fSES,lvf_last
          real,parameter :: adjust=0.942
          !
          type(runTime)T
          ! ===============
            !
            MYyear = T%year
            lvi_referenceYear=T%startyear-2
            k = MYyear-lvi_referenceYear
            !
           if(T%year == T%startyear)then
            do k = 1,6,1
             lvf_outSES(k)=lvf_in
            end do
           endif
            !
          if(lvf_Reduction_pct < 1)then
            alpha=1.
          else if(100 < lvf_Reduction_pct)then
            alpha = 0.45
          else
            if(lvf_Reduction_pct < 81)then
              lvf_Reduction_proportion=lvf_Reduction_pct * (1./100)
              alpha=a*b**lvf_Reduction_proportion * lvf_Reduction_proportion**c
              alpha=alpha*adjust
              alpha=alpha/100.     
            else
              if(81 < lvf_Reduction_pct)then
                if(lvf_Reduction_pct < 84)then
                  alpha=0.59
                else
                 if(lvf_Reduction_pct < 88)then
                  alpha=0.58
                 else
                  if(lvf_Reduction_pct < 91)then
                   alpha=0.55
                  else
                    alpha=0.45
                  endif
                 endif
                endif
              endif
            endif
            !
            if(year <=startYear)then
              fSES=lvf_outSES(1)
            else
              fSES= alpha*(lvf_outSES(k-1) &
                 +      (1.-alpha)*lvf_outSES(k-2) & 
                 + ((1.-alpha)**2)*lvf_outSES(k-3) &
                 + ((1.-alpha)**3)*lvf_outSES(k-4) &
                  )
            endif
            !
              if(lvf_last /= 0)then
               if(lvf_last < fSES)fSES=lvf_last
              endif
            !
          endif
              if(lvi_diffYear < T%year)fSES=lvf_outSES(76)
              if(k <=100)then
                lvf_outSES(k)=fSES
                lvf_last=fSES
              else
                lvf_outSES(k)=lvf_last
              endif
            !
        end subroutine sSES
        ! -----------------
!
! =================================================================================================
! E.O.F.




        ! ------------------------------------------
!        subroutine sIntendedUse(T,vPV,lv_PIU,i,vTstep)
!
!           integer :: i,j,k
!           integer :: vTstep
!           real :: lv_PIU(gvi_maxProV,gpi_timeStepWB)
!!           integer(2) :: lvf_GPCD_intended(gvi_maxProV,gpi_timeStepWB)
!
!           type(Provider)vPV
!           type(runTime)T
!           ! ===============
!            !
!            if(i == 1)then
!             if(vTstep == 1)then
!              do j = 1,gvi_Providers,1
!                do k = 1,12,1
!                 lv_PIU(j,k)=0
!                end do
!              end do
!             endif
!            endif
!            !
!!             lv_PIU(i,vTstep)=(vPV%gvf_pMunicipalPumping_acft_monthly(i,vTstep)+vPV%cSWsupply_acft(i,vTstep) &
!!                   -vPV%gvd_cGWrecharge_AF_monthly(i,vTstep))
!!            !
!            ! Not correct above ... 08.23.10 das
!!             lv_PIU(i,vTstep)=vPV%cSWandSRPused(i,vTstep)*0.945 ! leakage
!            !
!!              lvf_GPCD_intended(i,vTstep)= nint( (lv_PIU(i,vTstep) &
!!               * 1/gpf_daysMonth *gpd_galperacft *1./li_providerpop(T%year,i))*0.666 ) ! residential use
!
!            if( lv_PIU(i,vTstep) < 0)then
!              if(gvl_WriteToDisk)&
!               write(7,*)"Intended use < 0- City Water Budgets line 1565"
!            endif
!            !
!        end subroutine sIntendedUse
!        ! --------------------------
!
!        ! -------------------------------------------
!        subroutine sReclaimedUse(T,lvWB,lv_PIU,i,vTstep)
!          integer :: i,j,k
!           integer :: vTstep
!           real :: lv_PIU(gvi_maxProV,gpi_timeStepWB)
!!           integer(2) :: lvf_GPCD_realized(gvi_maxProV,gpi_timeStepWB)
!!           real :: lvf_sumResGray(gvi_maxProV,gpi_timeStepWB)
!           real :: lvf_AMA_grayWater(gpi_timeStepWB)
!           integer  :: lv_ProviderReclaimed(gvi_maxProV,gpi_timeStepWB)
!
!           type (waterbalance)lvWB
!!           type(Provider)vPV
!           type(runTime)T
!           ! ===============
!             !
!             k = vTstep
!              !
!             
!              do j = 1,gvi_Providers,1
!                lv_ProviderReclaimed(j,1)=0.
!                go_ProviderPostReclaimed(j)=0
!              end do
!              !
!!               lvf_GPCD_realized(i,vTstep)= nint(((vPV%cSWandSRPused(i,k)*0.666*0.945 )- lvWB%lv_fluxReclaimedGreywaterRes(i,k) ) &
!!               * 1/gpf_daysMonth *gpd_galperacft *1./li_providerpop(T%year,i) )
!
!              lv_ProviderReclaimed(i,k)=lvWB%lv_fluxReclaimedGreywaterRes(i,k)+lvWB%lv_fluxReclaimedGreywaterCIO(i,k)
!              !
!              if(vTstep == 1)then
!                go_ProviderGreyWReclaimed(i)=0
!                go_ProviderPostReclaimed(i)=0
!                
!              endif
!              !
!              lvf_AMA_grayWater(k)=lvf_AMA_grayWater(k)+lvWB%lv_fluxReclaimedGreywaterRes(i,k)
!              !
!            if(gpi_timeStepWB /=1)then
!                go_ProviderGreyWReclaimed_monthly(i,vTstep)=lv_ProviderReclaimed(i,vTstep)
!                go_ProviderGreyWReclaimed(i)= go_ProviderGreyWReclaimed(i)+lv_ProviderReclaimed(i,vTstep)
!                go_ProviderPostReclaimed(i)= go_ProviderPostReclaimed(i) +(nint(lv_PIU(i,vTstep)- lv_ProviderReclaimed(i,vTstep)))
!            else
!            endif
!              !
!        end subroutine sReclaimedUse
        ! --------------------------
